% Simulate the model with both anticipated and surprise information.
%
% m                  model or sirius_dynamic_model object
% rng                forecasting range
% din                input database with model lags filled in before the start of
%                    rng, and endogenous tunes (or tuned shocks) filled in over
%                    the rng
% endoexo_pairs      structure mapping each tuned endo variable to a shock
%                    which explains the endogenous tune
% anticipated_shocks list of shocks which are to be anticipated

function [d, r] = combined_forecast(m, rng, din, endoexo_pairs, ...
                                    anticipated_shocks)

  if isa(m, 'sirius_dynamic_model')
    enames = m.exo_names();
  else
    enames = get(m, 'enames');
  end
  surprise_shocks = setdiff(enames, anticipated_shocks);
  % set surprise_pairs to surprise shock mapped to empty string
  surprise_pairs = cell2struct(cell(1,length(surprise_shocks)),surprise_shocks,2);
  % set anticipated_pairs to aticipated shock mapped to empty string
  anticipated_pairs = cell2struct(cell(1,length(anticipated_shocks)),anticipated_shocks,2);
  % fillout surprise_pairs and anticipated_pairs according to endoexo_pairs
  flds = fieldnames(endoexo_pairs);
  for i=1:length(flds)
    endo = flds{i};
    exo = endoexo_pairs.(endo);
    if isempty(strmatch(exo, anticipated_shocks, 'exact'))
      surprise_pairs.(exo) = endo;
    else
      anticipated_pairs.(exo) = endo;
    end
  end
  
  % do the actual calculation
  r = cmop.combined_forecast_compact(m, rng, din, surprise_pairs, ...
                                     anticipated_pairs);
  % save other information
  r.range = rng;
  r.dbin = din;
  r.surprise_pairs = surprise_pairs;
  r.anticipated_pairs = anticipated_pairs;
  
  % get the last output dtabase as the final result
  d = r.dbouts{end};
end

