% calculate confidence bands: for linear model it takes stderrs from the
% smoother, for non-linear model, it simulates around the passed baseline

function dcbands = confbands(self, m, d, dstd, anticipated_shocks, opt, sc)

  if islinear(m) || opt.stochastic_simulations == 0
    dcbands = make_linear_confidence_bands(m, d, dstd, opt.confidence_levels);
    save(self.fetch_file_name('write','scenario_forecast_confbands',sc),...
         'dcbands');
  else
    % non-linear model
    msir = sirmexmod('iris_model', m, 'omit_measurements', true);

    % create surprise duplicates of the anticipated shocks
    msir.sdm_handle.duplicate_shocks(anticipated_shocks, 'surprise_');
    %- set variances for the new duplicates as for original shocks and
    %- set original anticipated shocks to zero
    [v,~] = msir.sdm_handle.get_vcov();
    vnew = struct();
    for i = 1:length(anticipated_shocks)
      if isfield(v, anticipated_shocks{i})
        vnew.(['surprise_' anticipated_shocks{i}]) = ...
            v.(anticipated_shocks{i});
        vnew.(anticipated_shocks{i}) = 0;
      end
    end
    msir.sdm_handle.set_vcov(vnew, struct());
    %- put zeros for the new shocks to database d
    for i = 1:length(anticipated_shocks)
      d.(['surprise_' anticipated_shocks{i}]) = ...
          tseries(opt.forecast_range, 0);
    end
    %- get all surprise shocks
    surprise_shocks = setdiff(msir.sdm_handle.exo_names(), anticipated_shocks);

    [f, stat] = self.fetch_file_name('read','scenario_forecast_confbands',sc);
    if stat
      load(f);
    else
    % run simulations
    [draws, failed_sims, needed_homotopy_steps, ~, cbands] = ...
        simulate_stochastic(msir, d, opt.forecast_range, ...
                            'npshock',length(opt.forecast_range), ...
                            'nsim',opt.stochastic_simulations,...
                            'surprise_shocks',{surprise_shocks},...
                            'clev', opt.confidence_levels);
    % and convert to the resulting database
    dcbands = dbbatch(cbands, '$0', 'cbands.$0.cf','classFilter', ...
                      'struct','fresh',true);
    % and save
    save(self.fetch_file_name('write','scenario_forecast_confbands',sc),...
         'failed_sims','needed_homotopy_steps','dcbands');
    end
  end
  
  % and postprocess
  dcbands = cmop.postprocess_database(dcbands, opt.rs_maturities);
end

function cfBands = make_linear_confidence_bands(m, mean, std, clev)
  cfBands = struct();
  xnames=get(m,'xnames');
  logVars=get(m,'log');

  for iter1=1:numel(xnames)
    rng = get(std.(xnames{iter1}),'range');
    if logVars.(xnames{iter1}) % cf bands for log variables (which has lognormal distribution)
      cf = norm_confbands(clev, log(std.(xnames{iter1})(rng)));
      for iter2=1:size(cf,2)
        cf(:,iter2)= exp(cf(:,iter2)+log(mean.(xnames{iter1})(rng)));
      end
    else
      cf = norm_confbands(clev, std.(xnames{iter1})(rng));
      for iter2=1:size(cf,2)
        cf(:,iter2)=cf(:,iter2)+mean.(xnames{iter1})(rng);
      end
    end
    cfBands.(xnames{iter1}) = tseries(rng,cf);
  end
end

function cf=norm_confbands(clev,forecastStd)
%function calculates confidence intervals specified in clev for normal
%distributions with zero mean and std. deviations stored in vector
%forecastStd.
%
%Inputs: clev ....................... confidence levels for example 
%                                     [0.90 0.70 0.50 0.30 0.1]. In this 
%                                     case cf will have 11 timeseries for
%                                     each field (variable) in forecastStd
%        forecastStd ................ vector of std. deviations of normal
%                                     distributions

  np=length(forecastStd);

  % precalculate the confidence intervals for normal(0,1)
  mult = zeros(1,length(clev));
  for iter=1:length(clev)
    mult(iter) = -sqrt(2)*erfcinv((1-clev(iter)));% norminv(p)=-sqrt(2)*erfcinv(2*p)
  end

  % create the confbands
  cf = zeros(np, 2*length(clev)+1);
  for iter=1:length(clev)
    cf(:,iter) = +mult(iter)*forecastStd;
    cf(:,end-iter+1) = -mult(iter)*forecastStd;
  end
end
