% expand description defined by the model using the FPAS mnemonics
% introducing the descriptions of the variables being introduced by
% cmop.postprocess_database()

function desc = expand_description(desc, ratesmat)

  % exchange rate
  [desc, ~, added] = dbbatch(desc, 'S$1', 'desc.$0', 'nameFilter','L_S(.*)');
  desc = change_unit_to(desc, added, 'level');
  [desc, ~, added] = dbbatch(desc, 'OBS_S$1', 'desc.$0', 'nameFilter','OBS_L_S(.*)');
  desc = change_unit_to(desc, added, 'level');
 
  % DOTA
  [desc, ~, added] = dbbatch(desc, 'DOTA_$1', 'desc.$0', 'nameFilter','DLA_(.*)');
  desc = change_unit_to(desc, added, '%, QoQ, @ar');
  [desc, ~, added] = dbbatch(desc, 'OBS_DOTA_$1','desc.$0', 'nameFilter','OBS_DLA_(.*)');
  desc = change_unit_to(desc, added, '%, QoQ, @ar');
 
  % DOT4
  [desc, ~, added] = dbbatch(desc, 'DOT4_$1', 'desc.$0', 'nameFilter','D4L_(.*)');
  desc = change_unit_to(desc, added, '%, YoY');
  [desc, ~, added] = dbbatch(desc, 'OBS_DOT4_$1', 'desc.$0', 'nameFilter','OBS_D4L_(.*)');
  desc = change_unit_to(desc, added, '%, YoY');
 
  % interest rates
  rsnames = fieldnames(ratesmat);
  desc = dbbatch(desc, 'AC_$0', 'desc.$0', 'nameList', rsnames);
  desc = dbbatch(desc, 'NC_$0', 'desc.$0', 'nameList', rsnames);
  desc = dbbatch(desc, 'OBS_AC_$0', 'desc.$0', 'nameList', rsnames);
  desc = dbbatch(desc, 'OBS_NC_$0', 'desc.$0', 'nameList', rsnames);
end

function desc = change_unit_to(desc, added, unit)

  for i = 1:length(added)
    dsc = desc.(added{i});
    pdsc = regexp(dsc, '(?<com>.*) +\((?<unit>.*)\) *', 'names');
    if length(pdsc) == 0
      % add unit
      dsc = [dsc ' (' unit ')'];
    else
      % change unit
      dsc = [pdsc.com ' (' unit ')'];
    end
    % put it to desc
    desc.(added{i}) = dsc;
  end

end
