% Get the filename for the file to be either run, or read or
% written. This provides a unique interface to filename conventions.
%
% synopsis
% [f, stat] = fetch_file_name(self, action, desc, ...)
%
%  self    this object
%  action  either 'run', 'read', 'write'
%  desc    string describing the file
%  f       returns full name of the file
%  stat    returns true, if the file exists

function [f, stat] = fetch_file_name(self, action, desc, varargin)

  switch desc
    case 'round_options'
      f = [self.dir filesep 'codes' filesep self.prefix 'round_options.m'];
    case 'model_mat'
      f = [self.dir filesep 'results' filesep self.name varargin{1} '_model.mat'];
    case 'model_xml'
      f = [self.dir filesep 'results' filesep self.name varargin{1} '_model.xml'];
    case 'observed_data'
      f = [self.dir filesep 'results' filesep self.name '_observed_data.csv'];
    case 'scenario_tunes'
      f = [self.dir filesep 'tunes' filesep self.prefix ...
           varargin{1} '_tunes.csv'];
    case 'scenario_history_mean'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_history_mean.csv'];
    case 'scenario_history_std'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_history_std.csv'];
    case 'scenario_forecast_std'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_forecast_std.csv'];
    case 'scenario_forecast_confbands'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_forecast_confbands.mat'];
    case 'scenario_history_shock_decomp_mat'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_history_shocks_decomp.mat'];
    case 'scenario_history_shock_decomp_pdf'
      f = [self.dir filesep 'reports' filesep self.name ...
           '_' varargin{1} '_history_shocks_decomp.pdf'];
    case 'user_filter_history'
      f = [self.dir filesep 'codes' filesep self.prefix ...
           'filter_history.m'];
    case 'user_report_history'
      f = [self.dir filesep 'codes' filesep self.prefix ...
           'report_history.m'];
    case 'historical_interpretation'
      f = [self.dir filesep 'reports' filesep self.name ...
           '_' varargin{1} '_historical_interpretation.pdf'];
    case 'define_scenario'
      f = [self.dir filesep 'codes' filesep self.prefix ...
           'define_scenario_' varargin{:} '.m'];
    case 'convert_other_freq'
      f = [self.dir filesep 'codes' filesep self.prefix ...
           'convert_other_freq.m'];
    case 'scenario_forecast_csv'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_forecast.csv'];
    case 'scenario_forecast_csv_otherfreq'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_forecast_' varargin{2} '.csv'];
    case 'scenario_forecast_mat'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_forecast.mat'];
    case 'define_forecast_report'
      f = [self.dir filesep 'codes' filesep self.prefix ...
           'define_forecast_report.m'];
    case 'scenario_forecast_pdf'
      if length(varargin) == 1
        f = [self.dir filesep 'reports' filesep self.name ...
             '_' varargin{1} '_forecast.pdf'];
      elseif length(varargin) == 3
        f = [self.dir filesep 'reports' filesep self.name ...
             '_' varargin{1} '_vrs_' varargin{2} '_' varargin{3} '_forecast.pdf'];
      else
        error(['Unsupported number of varargin for desc=' desc]);
      end
    case 'scenario_forecast_detailed_pdf'
      f = [self.dir filesep 'reports' filesep self.name ...
             '_' varargin{1} '_forecast_detailed.pdf']; 
    case 'scenario_xml'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_scenario_spec.xml'];
    case 'fcast_report_xml'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_report_spec.xml'];
    case 'data_monthly_csv'
      f = [self.dir filesep 'data' filesep self.prefix 'monthly.csv'];
    case 'data_quarterly_csv'
      f = [self.dir filesep 'data' filesep self.prefix 'quarterly.csv'];
    case 'data_yearly_csv'
      f = [self.dir filesep 'data' filesep self.prefix 'yearly.csv'];
    case 'user_observed_data'
      f = [self.dir filesep 'codes' filesep self.prefix 'observed_data.m'];
    case 'set_param'
      f = [self.dir filesep 'model' filesep self.prefix 'set_param.m'];
    case 'model_report'
      f = [self.dir filesep 'reports' filesep self.name varargin{1} '_model_report.pdf'];
    case 'analyze_model_mat'
      f = [self.dir filesep 'results' filesep self.name '_analyze_model.mat'];
    case 'analyze_model_pdf'
      f = [self.dir filesep 'reports' filesep self.name '_analyze_model.pdf'];
    case 'scenario_change_decomp_mat'
      f = [self.dir filesep 'results' filesep self.name ...
           '_' varargin{1} '_vrs_' varargin{2} '_' varargin{3} '_decomp_forecast_change.mat'];
    case 'scenario_change_decomp_pdf'
      f = [self.dir filesep 'reports' filesep self.name ...
           '_' varargin{1} '_vrs_' varargin{2} '_' varargin{3} '_decomp_forecast_change.pdf'];
    case 'historical_databases'
      f = [self.dir filesep 'codes' filesep self.prefix ...
           'historical_databases.m'];
    case 'historical_forecasts_mat'
      f = [self.dir filesep 'results' filesep self.name ...
           '_historical_forecasts.mat'];
    case 'historical_forecasts_pdf'
      f = [self.dir filesep 'reports' filesep self.name ...
           '_historical_forecasts.pdf'];
    case 'special_report_uncondstderr'
      f = [self.dir filesep 'reports' filesep self.name ...
           '_special_report_uncondstderr.pdf'];
    case 'special_report_lossfunction'
      f = [self.dir filesep 'reports' filesep self.name ...
           '_special_report_lossfunction.pdf'];
    otherwise
      error(['Unsupported desc=' desc]);
  end
  
  % check action
  if ~strcmp(action, 'read') && ~strcmp(action, 'write') ...
        && ~strcmp(action, 'run')
    error(['Wrong action=' action]);
  end

  % check existence
  stat = (2 == exist(f));
  
  % display the message
  if stat || strcmp(action, 'write')
    disp(sprintf('%-6s %s', [action, ':'], f));
  end
end
