% filter history using Kalman filter or user provided routine
%
%

function filterhistory(self, scenario)

  % get round options
  opt = self.get_round_options();

  % read the model: variable m
  load(self.fetch_file_name('read','model_mat',''));

  % get observed database
  dobs = dbload(self.fetch_file_name('read','observed_data'));
  dobs = cmop.preprocess_database(dobs, opt.rs_maturities);

  % get user tunes and overlay
  dtunes = dbload(self.fetch_file_name('read','scenario_tunes',scenario));
  dtunes = cmop.preprocess_database(dtunes, opt.rs_maturities);
  din = dboverlay(dobs, dtunes);
  % get model switches for history (not the scenario)
  if ~isstruct(opt.model_linear), swt = opt.model_switches; ...
  else swt = opt.model_switches.history; end
  
  % filter it
  if opt.use_kalman
    %- run iris kalman
    [~,out] = filter(m, din, opt.history_range,'data','smooth', ...
                     'relative',false);
                 
    %- and save it
    out.mean = cmop.postprocess_database(out.mean, opt.rs_maturities);
    dbsave(out.mean, self.fetch_file_name('write','scenario_history_mean',scenario));
    dbsave(out.std,  self.fetch_file_name('write','scenario_history_std',scenario));
    dout = out.mean;
   
    %- and decompose it to the shocks
%     if isfield(opt, 'histdecomp_groups') && length(opt.histdecomp_groups) > 0
%       [f fighs] = cmop.decomp_shocks(m, dout, opt.histdecomp_range,...
%                                      opt.histdecomp_variables,...
%                                      opt.histdecomp_groups,...
%                                      opt.histdecomp_grnames);
%       %-- save it
%       save(self.fetch_file_name('write','scenario_history_shock_decomp_mat',scenario),'f');
%       %-- report shock decomposition
%       x = report.new(['Historical shock decomposition report on scenario: ' scenario ...
%                       ', round: ' self.name]);
%       cm = get(m, 'comments');
%       for i = 1:length(fighs)
%         var = opt.histdecomp_variables{i};
%         tit = ['Shock decomposition of ' var ' (' cm.(var) ')'];
%         x.userfigure(tit, fighs{i}); 
%       end
%       x.publish(self.fetch_file_name('write','scenario_history_shock_decomp_pdf',scenario),...
%                 'display',false);
%       close all;
%     end
  else
    %- run user given function
    f = self.fetch_file_name('run','user_filter_history');
    [p,n,e] = fileparts(f);
    p = pathincr(p);
    clear(n);
    dout = eval([n '(m, swt, opt.history_range, ' ...
                 'dobs, dtunes);']);
    dout = cmop.postprocess_database(dout, opt.rs_maturities);
    %- and save it
    dbsave(dout, self.fetch_file_name('write','scenario_history_mean',scenario));
  end

  % report it
  x = report.new(['Historical interpretation report on scenario: ' scenario ...
                  ', round: ' self.name]);
  f = self.fetch_file_name('run','user_report_history');
  [p,n,e] = fileparts(f);
  p = pathincr(p);
  clear(n);
  x = eval([n '(x, m, swt, opt.history_report_range, ' ...
            'dobs, dtunes, dout);']);
  x = report_shock_contrib_to_loglik(x, m, dout, opt.history_range);
  x.publish(self.fetch_file_name('write','historical_interpretation',scenario),...
            'display',false);
end

function x = report_shock_contrib_to_loglik(x, m, d, rng)
  
  % initialize
  desc = get(m,'descript');
  elist = get(m,'eList');
  heading = {'Shock','Description','Stderr','Contrib to minus loglik'};

  % calculate the data
  A = cell(length(elist),4);
  for i = 1:length(elist)
    A{i,1} = elist{i};
    A{i,2} = desc.(elist{i});
    A{i,3} = m.(['std_' elist{i}]);
    if A{i,3} > 1e-10
      A{i,4} = sum((d.(elist{i})/A{i,3}).^2);
    else
      A{i,4} = 0;
    end
  end
  
  % add the array
  x.array(['Contributions of shocks to minus log likelihood over ' ...
           dat2char(rng(1)) '--' dat2char(rng(end))], A,...
           'heading', heading,'long=',true);
end