% run the historical forecasts over the given range, the dates in the
% range do not need to have the same frequency as the model
%
%

function histforecast(self, rng)
  % get round options
  opt = self.get_round_options();

  % read the model
  load(self.fetch_file_name('read','model_mat',''));
  endonames = get(m, 'xlist');

  % get the historical data for the given range
  f = self.fetch_file_name('run','historical_databases');
  [p,n,e] = fileparts(f);
  p = pathincr(p); 
  clear functions; % clear(n) not sufficient here to prevent crash
  hdbs = eval([n '(self, rng)']);
  
  % loop over each date: kalman filter it and simulate
  r = {};
  for i = 1:length(hdbs)
    % get range for the whole historical database
    hrng = dbrange(hdbs{i});
    % smooth it
    warning('off','IRIS:dbase');
    hdbs{i} = cmop.preprocess_database(hdbs{i}, opt.rs_maturities);
    [~,dbsmooth] = filter(m, hdbs{i}, hrng, 'meanonly', true);
    dbsmooth = cmop.postprocess_database(dbsmooth, opt.rs_maturities);
    warning('on','IRIS:dbase');
    % calculate rng for the forecast
    fmodel = freq(dbsmooth.(endonames{1}));
    dt = rng(i);
    dtm = convert(dt, fmodel);
    np = opt.histfcast_periods;
    frng = dtm:dtm+np-1;
    % forecast 
    dbfcast = simulate(m, dbsmooth, frng, 'anticipate', false, ...
                       'dboverlay', true);
    dbfcast = cmop.postprocess_database(dbfcast, opt.rs_maturities);
    % save it to r
    r{end+1} = struct(...
        'hrng',     hrng,...
        'dt',       dt,...
        'dtm',      dtm,...
        'frng',     frng,...
        'dbsmooth', dbsmooth,...
        'dbfcast',  dbfcast,...
        'dbhist',   hdbs{i});
  end
  
  % save the r
  save(self.fetch_file_name('write','historical_forecasts_mat'), 'r');

  % report
  x = report.new(['Historical forecasts round: ' self.name]);
  x = cmop.process_histforecast_report(x, m, r, opt);
  x.publish(self.fetch_file_name('write','historical_forecasts_pdf'),...
            'display',false);
  
end
