% this processes database and according to the agreed mnemonics it
% transforms model internal units to external units performing the following
% transformations
%
% Model variable | User         | Note
% -----------------------------------------------------------
% L_S(.*)        | S\1          | Nominal exchange rate level
% DLA_(.*)       | DOTA_\1      | QoQ growth rate in per cent
% D4L_(.*)       | DOT4_\1      | YoY growth rate in per cent
% interest       | AC_interest  | Annually compounded interest (%, pa.)
% interest       | NC_interest  | Non-compounded interest (%, p.a.)
%

function d = postprocess_database(d, ratesmat)

  % exchange rate
  d = dbbatch(d, 'S','exp(d.$0/100)','nameFilter','^L_S$');
  d = dbbatch(d, 'OBS_S','exp(d.$0/100)','nameFilter','^OBS_L_S$');
  d = dbbatch(d, 'S_$1','exp(d.$0/100)','nameFilter','^L_S_(.*)');
  d = dbbatch(d, 'OBS_S_$1','exp(d.$0/100)','nameFilter','^OBS_L_S_(.*)');
  
  % DOTA
  d = dbbatch(d, 'DOTA_$1','100*(exp(d.$0/100)-1)','nameFilter','^DLA_(.*)');
  d = dbbatch(d, 'OBS_DOTA_$1','100*(exp(d.$0/100)-1)','nameFilter','^OBS_DLA_(.*)');
  
  % DOT4
  d = dbbatch(d, 'DOT4_$1','100*(exp(d.$0/100)-1)','nameFilter','^D4L_(.*)');
  d = dbbatch(d, 'OBS_DOT4_$1','100*(exp(d.$0/100)-1)','nameFilter','^OBS_D4L_(.*)');
  
  % interest rates
  rsnames = fieldnames(ratesmat);
  for i = 1:length(rsnames)
    rs = rsnames{i};
    ors = ['OBS_' rs];
    acrs = ['AC_' rs];
    ncrs = ['NC_' rs];
    oacrs = ['OBS_AC_' rs];
    oncrs = ['OBS_NC_' rs];
    m = ratesmat.(rs);
    if isfield(d, rs)
      d.(acrs) = 100*exp(d.(rs)/100-1);
      d.(ncrs) = 100/m*(exp(m*d.(rs)/100)-1);
    end
    if isfield(d, ors)
      d.(oacrs) = 100*exp(d.(ors)/100-1);
      d.(oncrs) = 100/m*(exp(m*d.(ors)/100)-1);
    end
  end
end
