% this processes database and according to the agreed mnemonics it
% transforms user external units to model internal units perfoming the
% following transformations
%
% Model variable | User         | Note
% -----------------------------------------------------------
% L_S            | S            | Nominal exchange rate level
% L_S_(.*)       | S_\1         | Nominal exchange rate level
% DLA_(.*)       | DOTA_\1      | QoQ growth rate in per cent
% D4L_(.*)       | DOT4_\1      | YoY growth rate in per cent
% interest       | AC_interest  | Annually compounded interest (%, pa.)
% interest       | NC_interest  | Non-compounded interest (%, p.a.)
%

function d = preprocess_database(d, ratesmat)

  % exchange rate
  d = dbbatch(d, 'L_S','100*log(d.$0)','nameFilter','^S$');
  d = dbbatch(d, 'OBS_L_S','100*log(d.$0)','nameFilter','^OBS_S$');
  d = dbbatch(d, 'L_S_$1','100*log(d.$0)','nameFilter','^S_(.*)');
  d = dbbatch(d, 'OBS_L_S_$1','100*log(d.$0)','nameFilter','^OBS_S_(.*)');
  
  % DOTA
  d = dbbatch(d, 'DLA_$1','100*log(1+d.$0/100)','nameFilter','^DOTA_(.*)');
  d = dbbatch(d, 'OBS_DLA_$1','100*log(1+d.$0/100)','nameFilter','^OBS_DOTA_(.*)');
  
  % DOT4
  d = dbbatch(d, 'D4L_$1','100*log(1+d.$0/100)','nameFilter','^DOT4_(.*)');
  d = dbbatch(d, 'OBS_D4L_$1','100*log(1+d.$0/100)','nameFilter','^OBS_DOT4_(.*)');
  
  % interest rates
  rsnames = fieldnames(ratesmat);
  for i = 1:length(rsnames)
    rs = rsnames{i};
    ors = ['OBS_' rs];
    acrs = ['AC_' rs];
    ncrs = ['NC_' rs];
    oacrs = ['OBS_AC_' rs];
    oncrs = ['OBS_NC_' rs];
    m = ratesmat.(rs);
    if isfield(d, acrs)
      d.(rs) = 100*log(1+d.(acrs)/100);
    end
    if isfield(d, ncrs)
      d.(rs) = 100/m*log(1+m*d.(ncrs)/100);
    end
    if isfield(d, oacrs)
      d.(ors) = 100*log(1+d.(oacrs)/100);
    end
    if isfield(d, oncrs)
      d.(ors) = 100/m*log(1+m*d.(oncrs)/100);
    end
  end
end
