% add to the report object the pages defined by the report definition
%
%
% x            report object
% m            model object
% dbsmean      databases with means (one or two, first is the scenario, second is the
%              control)
% dbsmean2     databases with means for the second part of the table 
% dbsstd       database with stds (one or two, first is the scenario, second is the
%              control)
% dcbands      database with confidence bands for the scenario
% labs         labels for the databases
% rd           report definition
% opt          round options

function x = process_forecast_report(x, m, dbsmean, dbsmean2, dbsstd, dcbands, labs, rd, opt)

  % go through the pages in the report definition
  for i = 1:length(rd)
    if isfield(rd{i}, 'table')
      % page has a table
      tbd = convert_table_def(rd{i}, m, opt.rs_maturities);
      params = {'rng', opt.report_table_range,...
                'col1width', opt.report_table_col1width,...
                'ds2', dbsmean2};
      if isfield(rd{i}, 'titlefootnote')
        params = {params{:}, 'titfootnote', rd{i}.titlefootnote};
      end
      if isfield(opt, 'report_table_range2')
        params = {params{:}, 'rng2', opt.report_table_range2};
      end
      if isfield(rd{i}, 'show_std') && rd{i}.show_std ...
            && length(dbsstd) > 0 && length(dbsmean) < 2
        params = {params{:}, 'db_std', dbsstd{1}};
      end
      x = add_diff_table(x, rd{i}.title, dbsmean, labs, tbd, ...
                         params{:});
    else
      % page has the graphs
      x = add_graphs(x, m, rd{i}, dbsmean, dcbands, labs, opt);
    end
    x.pagebreak();
  end

%   % add tables of shocks
%   enames = get(m, 'elist');
%   n = ceil(length(enames)/16);
%   nn = ceil(length(enames)/n);
%   for i = 1:n
%     si = (i-1)*nn+1;
%     ei = min(length(enames), i*nn);
%     tbd = shocks_table_def(enames(si:ei), m);
%     x = add_diff_table(x, ['Shocks ' num2str(i)], dbsmean, labs, tbd,...
%                        'rng', opt.report_table_range, ...
%                        'col1width', opt.report_table_col1width);
%     x.pagebreak();
%   end
  
  % custom: add the REER gap and trend chart
  db = dbsmean{1};
  fh = figure('visible','off');
  subplot(211);
    p = plot(qq(2010,1):qq(2017,4), [db.l_z db.l_z_tnd],...
      'LineWidth',3);
    set(p(2),'LineStyle','--');
    grid;
    axis tight;
    box on;
    g = grfun.bottomlegend({'Level','Trend'}, 'location','SouthOutside','orientation', ...
      'horizontal');
    p = get(g, 'position');
    set(g, 'position',[p(1),p(2)+0.03,p(3),p(4)]);
    title('Real Effective Exchange Rate');
  subplot(212);
    p = plot(qq(2010,1):qq(2017,4), [db.l_qoil db.l_qoil_tnd],...
      'LineWidth',3);
    set(p(2),'LineStyle','--');
    grid;
    axis tight;
    box on;
    g = grfun.bottomlegend({'Level','Trend'}, 'location','SouthOutside','orientation', ...
      'horizontal');
    p = get(g, 'position');
    set(g, 'position',[p(1),p(2)+0.03,p(3),p(4)]);  
    title('Real Oil Price');
  x.userfigure('Real Effective Exchange Rate', fh);
  
end


% convert simple table definition to the table definition good for add_diff_table 
function tbd = convert_table_def(rdi, m, rsmat)
  desc = cmop.expand_description(get(m, 'description'), rsmat);
  islog = cmop.expand_islog(get(m, 'log'), rsmat);
  
  tbd = struct();
  tbd.linekeys = cell(1,0);
  ii = 1;
  for i = 1:length(rdi.table)
    key = ['k' num2str(ii)]; ii = ii+1;
    tbd.linekeys{end+1} = key;
    tbd.(key).label = rdi.table{i}.label;
    tbd.(key).indent = 0;
    for j = 1:length(rdi.table{i}.series)
      var = rdi.table{i}.series{j};
      dsc = desc.(var);
      pdsc = regexp(dsc, '(?<com>.*) +\((?<unit>.*)\) *', 'names');
      if length(pdsc) == 0
        pdsc = struct('com',dsc,'unit','');
      end
      key = ['k' num2str(ii)]; ii = ii+1;
      tbd.linekeys{end+1} = key;
      tbd.(key).label = pdsc.com;
      if isfield(rdi.table{i},'serieslabs')
        tbd.(key).label = rdi.table{i}.serieslabs{j};
      end
      tbd.(key).varname = var;
      tbd.(key).varunit = strrep(pdsc.unit, '%', '\%');
      tbd.(key).indent = 1;
      tbd.(key).dev = 'pp';
      if islog.(var), tbd.(key).dev = 'logdiff'; end;
      tbd.(key).dd = 1;
      if isfield(rdi.table{i},'dd'), tbd.(key).dd = rdi.table{i}.dd{j}; end;
      tbd.(key).sd = 4;
    end
  end
end

% create table definition for the provided list of shocks of the provided model
function tbd = shocks_table_def(enames, m)

  desc = get(m, 'description');
  tbd = struct();
  tbd.linekeys = {};
  for ii = 1:length(enames)
    key = ['k' num2str(ii)];
    tbd.linekeys{end+1} = key;
    dsc = desc.(enames{ii});
    pdsc = regexp(dsc, '(?<com>.*) +\((?<unit>.*)\) *', 'names');
    if length(pdsc) == 0
      pdsc = struct('com',dsc,'unit','');
    end
    tbd.(key).label = pdsc.com;
    tbd.(key).varname = enames{ii};
    tbd.(key).varunit = strrep(pdsc.unit, '%', '\%');
    tbd.(key).indent = 0;
    tbd.(key).dev = 'pp';
    tbd.(key).dd = 1;
    tbd.(key).sd = 4;
  end
end


% adds diff table to the report
% for a definition of tbd and varargin, look at make_diff_latex_table.m
% varargin accepts here one option: fontSize is a string of LaTeX size
% (defaults to '\tiny')

function x = add_diff_table(x, tit, dbs, labs, tbd, varargin)
  
  opt = struct();
  opt.fontSize	= '\scriptsize';%'\tiny';
  opt.tunes = 0;
  opt.marks = 0;
  opt.titfootnote = 0;
  opt.db_std = 0;
  for iter=1:2:length(varargin)
    switch lower(varargin{iter})
      case 'savetex'
        opt.savetex  = varargin{iter+1};
      case 'fontsize'
        opt.fontSize = varargin{iter+1};
      case 'db_tune'
        if sum(isfield(varargin{iter+1},{'tunes','hist','obs'})) ~= 3
           warning('To use tunes in comparison report, specify proper database with all required fields.');
        else
            opt.tunes = 1;
        end
      case 'db_mark'
        opt.marks = 1;
      case 'titfootnote'
        opt.titfootnote = 1;
        opt.titfootnotetext = varargin{iter+1};
      case 'db_std'
        opt.db_std = 1;
    end
  end
  

  % calculate raw table
  rawtable = make_diff_latex_table(dbs, tbd, varargin{:});
  
  % footnote
  nstrs = {'first','second','third','fourth','fifth','sixth','seventh',...
           'eighth','nineth', 'tenth'};
  fttext = ['Baseline: {\bf ' labs{1} '}'];
  for i=2:length(dbs)
    fttext = [fttext '; ' nstrs{i-1} ' line: {\bf ' labs{i} '}'];
  end
  if opt.tunes
    fttext = [fttext '. Hard data in bold, tunes for the forecast in bold with star'];
  end
  
  % pagelayout
  nl = char(10);
  out = ['\hbox to\hsize{\hss\Large ' tit '\footnote{' fttext '}'];
  if length(dbs) > 1
    out = [out '\ \footnote{Deviations in (percent) or in [percentage points]}' nl];
  elseif opt.db_std
    out = [out '\ \footnote{Std. deviations in (percent) or in [simple difference]}' nl];
  end
  if opt.titfootnote
      out = [out '\ \footnote{' opt.titfootnotetext '}' nl];
  end
  out = [out '\hss}' nl '\vskip 4mm' nl];
  out = [out '\hbox to\hsize{\hss\hbox{%' nl];
  out = [out opt.fontSize];
  out = [out rawtable];
  out = [out '\normalsize'];
  out = [out '}\hss}' nl];
  
  % put to the report
  x.tex('',out);
  
  % save to the tex file
  if isfield(opt, 'savetex')
    fid = fopen(opt.savetex,'w');
    fprintf(fid, '%s', rawtable);
    fclose(fid);
  end
end

% makes a latex table comparing ds{1} with ds{2:end} databases with the
% given specification in tbd and returns it as a string; if 'fcast' option
% is given, then it displays the data in the fcast option with italics
%
% author: Ondra Kamenik
%
% out = make_diff_latex_table(ds, tbd, varargin)
%
% ds are databases, ds{1} is a baseline, ds{2:end} are displayed as d1
% minus a database
%
% tbd.linekeys = cell array (row) of line keys
% tbd.<linekey> describes the line
% tbd.<linekey>.label string for line lable
% tbd.<linekey>.indent = indetation level of the line
% tbd.<linekey>.dev = p|pp|none
%   'logdiff' deviations are displayed in (100*log(baseline/database))
%   'pp' deviations are displayed as percentage points [baseline-database]
%   'subtract' ordinary difference as baseline-database
% tbd.<linekey>.varname = variable name in the database (if empty, or not
%   given, nothing is printed)
% tbd.<linekey>.sd = number of significant digits for rounding
% tbd.<linekey>.varunit = units that variable is defined in (if empty, or not
%   given, nothing is printed)
% tbd.<linekey>.labelfontsize = tiny|scriptsize|footnotesize|small|normalsize|large|Large|LARGE|huge|Huge ...
%   (string of LaTeX size)
%
% from varargin:
% rng           range for the periods (no default)
% rng2          range for the secondary range (defaults to empty)
% ds2           databases for the secondary range referenced if rng2 is
%               not empty (no default)
% fcast         if given, it expects cell array of databases of the same
%               length as ds and these data is displayed in italics
% colwidth      TeX dimension string for column witdh; (defaults to '8mm')
% col1width     TeX dimension string for the first column witdh (defaults to
%               '34mm')
% rowheight     TeX dimension for the row height (defaults to 14pt)
% devrowheight  TeX dimension for the deviation line row height (defaults
%               to 7pt)
% tabcolsep	    Tex dimension for the space between two columns (defaults to 0.4mm)
% db_tune       if given, it expects database of 3 subdatabases with these names:
%               'tunes' (with tunes only for fcast) displayed with star, 'hist' 
%               (only hard data) displayed in bold, 'obs' (all model observables)
% db_mark       if given, it expects a database with subdatabase 'marks' (with whatever tunes 
%               or other marks for the timeseries) displayed in bold with caret ('^')
% db_std        if given, then deviations are not printed but the second
%               line will have standard errors 
% 


function out = make_diff_latex_table(ds, tbd, varargin)

  % get options
  optnames = {varargin{1:2:end}};
  optvals = {varargin{2:2:end}};
  opt = cell2struct(optvals, optnames, 2);

  if ~isfield(opt, 'colwidth')
    opt.colwidth = '8mm';
  end
  if ~isfield(opt, 'col1width')
    opt.col1width = '34mm';
  end
  if ~isfield(opt, 'rowheight')
    opt.rowheight = '10pt';
  end
  if ~isfield(opt, 'devrowheight')
    opt.devrowheight = '6pt';
  end
  if ~isfield(opt, 'emptystr')
    opt.emptystr = '';
  end
  if ~isfield(opt, 'tabcolsep')
    opt.tabcolsep = '0.4mm';
  end
  if ~isfield(opt, 'rng2')
    opt.rng2 = [];
  end
  if ~isfield(opt, 'db_tune')
    opt.db_tune = struct('tunes',struct(),'hist',struct(),'obs',struct());
  end
  if ~isfield(opt, 'db_mark')
    opt.db_mark = struct('marks',struct());
  end
  % set dsf
  dsf = {};
  for i=1:length(ds)
    dsf{end+1} = struct();
  end
  if isfield(opt, 'fcast')
    dsf = opt.fcast;
  end
  % set ds2
  if isfield(opt, 'ds2')
    ds2 = opt.ds2;
  else
    ds2 = {};
    for i = 1:length(ds)
      ds2{end+1} = struct();
    end
  end
  % set dsf2
  dsf2 = {};
  for i=1:length(ds)
    dsf2{end+1} = struct();
  end
  % set opt.db_tune2 and opt.db_mark2
  opt.db_tune2 = struct('tunes',struct(),'hist',struct(),'obs',struct());
  opt.db_mark2 = struct('marks',struct());


  
  
  out = '';
  nl = char(10);
  nanstr = ['\hbox to' opt.colwidth ' {\hss ' opt.emptystr '}'];

  % phantoms
  normal_phantom = ['\hbox{\vrule width0pt height',opt.rowheight,' depth3pt}'];
  pdev_phantom = ['\hbox{\vrule width0pt height',opt.devrowheight,' depth1pt}'];

  % the number of data columns
  nc = length(opt.rng);
  nc2 = length(opt.rng2);
  
  % the frequencies of the ranges
  [~,~,freq] = dat2ypf(opt.rng(1));
  freq2 = NaN; if nc2 > 0, [~,~,freq2] = dat2ypf(opt.rng2(1)); end
  
  % header line
  out = [out '\renewcommand{\tabcolsep}{' opt.tabcolsep '}' nl];
  sepc = '|'; if nc == 0 || nc2 == 0, sepc = ''; end
  out = [out '\begin{tabular}{l' repmat('r',1,nc) sepc repmat('r',1,nc2) '}' nl];

  % first line only years
  out = [out normal_phantom];
  out = [out latex_table_year_range(opt.rng) latex_table_year_range(opt.rng2)];
  out = [out '\\' nl];

  % second line for periods within a year
  if freq > 1 || (nc2 > 0 && freq2 > 1)
    out = [out normal_phantom];
    out = [out latex_table_period_range(opt.rng, opt.colwidth)...
           latex_table_period_range(opt.rng2, opt.colwidth)];
    out = [out '\\' nl];
  end

  % write all other lines
  for j = 1:length(tbd.linekeys)
    lkey = tbd.linekeys{j};

    % if tbd.(lkey) is missing, treat lkey as a variable name
    if ~isfield(tbd, lkey)
      tbd.(lkey).label = strrep(lkey, '_', '\_');
      tbd.(lkey).indent = 1;
      tbd.(lkey).dev = 'pp';
      tbd.(lkey).varname = lkey;
      tbd.(lkey).sd = 3;
      tbd.(lkey).dd = 1;
    end
    
    out = [out normal_phantom]; 
    % do label
    ind = 0;
    out = [out '\hbox to ' opt.col1width '{'];
    if isfield(tbd.(lkey), 'indent')
      ind = tbd.(lkey).indent;
      out = [out '\hskip ' num2str(ind) 'em '];
    end
    if isfield(tbd.(lkey), 'label')
       if isfield(tbd.(lkey), 'labelfontsize')
          out = [out '{\' tbd.(lkey).labelfontsize ' ' tbd.(lkey).label '}'];
       else
          out = [out '{' tbd.(lkey).label '}'];
       end
    end
    %- add unit if it is not going to be added on the second line
    if length(ds) == 1 && isfield(tbd.(lkey),'varunit')
      out = [out ' (' tbd.(lkey).varunit ')'];
    end
    %- finish the hbox
    out = [out '\hss}'];

    if isfield(tbd.(lkey),'varname')
      vname = tbd.(lkey).varname;

      % do the baseline
      out1 = latex_table_baseline_data(ds, dsf, tbd.(lkey), opt.rng, opt.db_tune, ...
                                       opt.db_mark, nanstr, opt.colwidth);
      out2 = latex_table_baseline_data(ds2, dsf2, tbd.(lkey), opt.rng2, opt.db_tune2, ...
                                       opt.db_mark2, nanstr, opt.colwidth);
      out = [out out1 out2];

      % do the deviation lines
      for i=2:length(ds)
        % finish the previous line
        out = [out '\\[-1mm]' nl];
          
        out = [out pdev_phantom];
        
        % add unit description
        if i==2 && isfield(tbd.(lkey),'varunit')
          out = [out '\hbox to ' opt.col1width '{'];
          out = [out '\hskip ' num2str(ind) 'em '];
          out = [out '(' tbd.(lkey).varunit ')\hss}'];
        end
     
        out1 = latex_table_deviation_data(i, ds, dsf, tbd.(lkey), opt.rng, ...
                                          nanstr, opt.colwidth);
        out2 = latex_table_deviation_data(i, ds2, dsf2, tbd.(lkey), opt.rng2, ...
                                          nanstr, opt.colwidth);
        out = [out out1 out2];
      end %- end of deviation lines
      
      % print the standard errors
      if length(ds) < 2 && isfield(opt, 'db_std');
        % finish the previous line
        out = [out '\\[-1mm]' nl];
        
        out = [out pdev_phantom];
        
        % add unit description
        out = [out '\hbox to ' opt.col1width '{'];
        out = [out '\hskip ' num2str(ind) 'em '];
        out = [out 'std. deviation\hss}'];
     
        out1 = latex_table_stderr_data(opt.db_std, tbd.(lkey), opt.rng, nanstr, ...
                                       opt.colwidth);
        out2 = latex_table_stderr_data(struct(), tbd.(lkey), opt.rng2, nanstr, ...
                                       opt.colwidth); %- no stderrors for rng2
        out = [out out1 out2];
      end %- end of stderr lines
        
    else %- of isfield(tbd.(lkey),'varname')
      out = [out repmat('&',1,nc+nc2)];
    end %- of isfield(tbd.(lkey),'varname')
    
    % finish the block with baseline and deviations
    out = [out '\\' nl];
  end

  % finish the table
  out = [out '\end{tabular}' nl];

end

% return latex table header code for years for a given range, depending
% on the underlying frequency, we need to put them to multicolumns
function out = latex_table_year_range(rng)
  out = '';
  if length(rng) > 0
    [~,~,freq] = dat2ypf(rng(1));
    for iy = dat2ypf(rng(1)):dat2ypf(rng(end));
      ybegin = datcode(freq, iy, 1);
      yend = datcode(freq, iy, freq);
      cols = length(max(ybegin,rng(1)):min(yend,rng(end)));
      ystr = num2str(iy);
      if cols < 2 && freq > 1 % skip printing year if only one column
        ystr = '';
      end
      algn = 'r';  if freq ~= 1, algn = 'c'; end
      out = [out '&\multicolumn{' num2str(cols) '}{' ...
             algn '}{' ystr '}'];
    end
  end
end

function out = latex_table_period_range(rng, colwidth)
  out = '';
  if length(rng) > 0
    [~,~,freq] = dat2ypf(rng(1));
    for rd = rng
      perchar = dat2char(rd,'dateformat','FP');
      if freq == 1, perchar = ''; end
      out = [out '&\vbox{\hbox to ' colwidth ...
             '{\hss ' perchar '}\hrule width ' colwidth '}'];
    end
  end
end

% create string for the content of the first line (absolute values) for
% the given range
function out = latex_table_baseline_data(ds, dsf, tbdlkey, rng, dbtune, ...
                                         dbmark, nanstr, colwidth);
  out = '';
  vname = tbdlkey.varname;
  
  [data,ifcast] = tseries2data(ds{1}, dsf{1}, tbdlkey.varname, rng);
  if any(~isnan(data))
    for k = 1:length(rng)
      if isnan(data(k))
        f = nanstr;
      else
        f = num2gstr(data(k), tbdlkey.sd, tbdlkey.dd);
      end
      f = regexprep(f, '^0\.00', '.00');
      if length(f) > 5
        f = regexprep(f, '^0.0', '\.0');
      end
      if ifcast(k)
        f = ['\it ' f ];
      end
      if isfield(dbtune.hist,vname) && ~isnan(dbtune.hist.(vname)(rng(1)+k-1))
        f = ['\bf ' f ];
      end
      if isfield(dbtune.tunes,vname) &&  ~isnan(dbtune.tunes.(vname)(rng(1)+k-1))
        f = ['\bf' f '\rlap{*}' ];
      end
      if isfield(dbmark.marks,vname) && ~isnan(dbmark.marks.(vname)(rng(1)+k-1))
        f = ['\bf ' f '\rlap{\^}' ];
      end
      out = [out '&\hbox to ' colwidth '{\hss ' f '}'];
    end
  else
    out = [ out repmat('&',1,length(rng))];
  end
end


function out = latex_table_deviation_data(i, ds, dsf, tbdlkey, rng, nanstr, ...
                                          colwidth)

  vname = tbdlkey.varname;
  
  % baseline data  
  [data,ifcast] = tseries2data(ds{1},dsf{1},vname,rng);
  % deviation data
  [datacmp,ifcastcmp] = tseries2data(ds{i},dsf{i},vname,rng);
  % do deviations?
  dodev = any(~isnan(data)) && isfield(tbdlkey, 'dev') && ...
          (strcmp(tbdlkey.dev, 'logdiff') || strcmp(tbdlkey.dev, 'pp') || ...
           strcmp(tbdlkey.dev, 'subtract'));
  % create the output
  out = '';
  for k = 1:length(rng)
    if isnan(datacmp(k)) || isnan(data(k))
      f = nanstr;
    elseif dodev
      f = diff2gstr(data(k), datacmp(k), ...
                    tbdlkey.sd, tbdlkey.dd, ...
                    tbdlkey.dev);
      % replace the first bracket with 'bracket\hss' if any bracket
      % otherwise add \hss at the beginning of the string
      f = regexprep(f, '([^ \[\(\]\)]*)','\\hss $1');
    else
      f = num2gstr(datacmp(k), tbdlkey.sd, tbdlkey.dd);
    end
    if ifcast(k) || ifcastcmp(k)
      f = ['\it ' f ];
    end
    % and put to out
    out = [out '&' '\hbox to ' colwidth '{\relsize{-1} ' f '}'];
  end
end

function out = latex_table_stderr_data(db_std, tbdlkey, rng, nanstr, ...
                                       colwidth);
  out = '';
  % do the data for the deviation
  [datacmp,ifcastcmp] = tseries2data(opt.db_std,struct(),tbdlkey.varname,rng);
  for k = 1:length(rng)
    if isnan(datacmp(k)) || isnan(data(k))
      f = nanstr;
    else
      f = std2gstr(datacmp(k), tbdlkey.sd, tbdlkey.dd, tbdlkey.dev);
      % replace the first bracket with 'bracket\hss' if any bracket
      % otherwise add \hss at the beginning of the string
      f = regexprep(f, '([^ \[\(\]\)]*)','\\hss $1');
    end
    % and put to out
    out = [out '&' '\hbox to ' opt.colwidth '{\relsize{-1} ' f '}'];
  end
end

% return data vector having a a union of data from db1, db2 of the
% varname at the given range rng, and return isecond as a logical array
% having trues at the positions at which the data were filled from db2
% and were not found in db1
function [data, isecond] = tseries2data(db1, db2, varname, rng)
  
  data1 = nan(length(rng),1);
  if isfield(db1, varname)
    data1 = db1.(varname)(rng);
  end
  data2 = nan(length(rng),1);
  if isfield(db2, varname)
    data2 = db2.(varname)(rng);
  end
  
  % merge data2 into data1
  data = data1;
  isecond = isnan(data1) & ~isnan(data2);
  data(isecond) = data2(isecond);
end

% converts a double to a string with a given number of significant digits
% and maximum number of decimal digits
function str = num2gstr(a, sd, dd)
  
  if a == 0
    str = '0';
    return;
  end
  
  m = ceil(log(abs(a))/log(10));
  sd = min(sd, m+dd);
  g = sign(a)*round(abs(a)*10^(sd-m))/10^(sd-m);

  % if g is zero or significant digits are zero, exit with zero
  if a == 0 || g == 0 %|| sd == 0
    str = '0';
    return;
  end
  
  f='%f';
  % make a format and accommodate g
  if m < sd & m > -2
    f = sprintf('%%%d.%df',max(m-1)+1,sd-m);
  elseif m == sd | m == sd+1
    f = '%d';
  elseif m > sd+1
    f = sprintf('%%1.%dfe%d',max(0,sd-1),m-1);
    g = g*10^(-m+1);
  elseif m <= -2
    f = sprintf('%%1.%dfe%d',max(0,sd-1),m-1);
    g = g*10^(-m+1);
  end
  % format and return
  str = sprintf(f, g);
end

% converts difference b from a both with sd significant digits and dd
% decimal digits of type 'type' to the string representation; type can be
% either 'subtract', 'logdiff', or 'pp'. 'pp' and 'subtract' are
% mathematically equivalent

function str = diff2gstr(a, b, sd, dd, type)

  if abs(a-b)<1e-10 % used tolerance, originally: a == b
    str = '=';
    return;
  end
  
  str = difference2gstr(a, b, sd, dd, type);
end

function str = std2gstr(s, sd, dd, type)
  if strcmp('logdiff',type)
    b = 1;
    a = b*s;
  else
    b = 0;
    a = b + s;
  end
  str = difference2gstr(a, b, sd, dd, type);
end

function str = difference2gstr(a, b, sd, dd, type)
  % calculate significant digits for a difference
  sdd = max(0,sd-(ceil(log(abs(a))/log(10))-ceil(log(abs(a-b))/log(10))));
  
  % make the difference
  g = a-b;
  if strcmp('logdiff',type)
    g = 100*log(a/b);
  end

  % set left and right brackets
  lb = '';
  rb = '';
  if strcmp('pp',type)
    lb = '[';
    rb = ']';
  elseif strcmp('logdiff', type)
    lb = '(';
    rb = ')';
  end
  
  % return the string
  str = [lb num2gstr(g, sdd, dd) rb];
end

% add graphs to the current page of the report
function x = add_graphs(x, m, rdi, dbs, dcbands, labs, opt)

  tit = rdi.title;
  series = rdi.series;
  nx = 1 + (length(series)>1);
  ny = ceil(length(series)/nx);
  desc = cmop.expand_description(get(m, 'descript'), opt.rs_maturities);
  ss = cmop.postprocess_database(get(m, 'sstate'), opt.rs_maturities);
  
  % style
  st = struct();
  st.line.lineWidth = {2,2};
  st.line.lineStyle = {'-','--'};
  st.line.color = {[0.8,0,0],[0.1,0.1,0.1]};
  st.line.marker = {'none','none'};
  
  % make a string of legends
  mss = sprintf('''%s'',', labs{:});
  mss = mss(1:end-1);

  fh = figure('visible','off');
  for i = 1:length(series)
    zline = abs(ss.(series{i})) < 1e-5;
    subplot(nx, ny, i);
    if i == length(series)
      % for the last graph do it also before shaded plot because of the legend
      for j = 1:length(dbs)
        jj = rem(j-1,2)+1;
        plot(opt.report_plot_range, dbs{j}.(series{i}),...
             'LineWidth',st.line.lineWidth{jj},...
             'LineStyle',st.line.lineStyle{jj},...
             'Color',st.line.color{jj}); hold on;
      end
    end
    if zline
      plot(opt.report_plot_range, tseries(opt.report_plot_range,0), 'Color','black');hold on;
    end
    if isfield(rdi, 'confbands') && rdi.confbands
      cbands_rng = intersect(get(dcbands.(series{i}),'range'),opt.report_plot_range);
      bb = dcbands*series;
      dbsave(bb, ['~/Desktop/' labs{:} '.csv']);
      if isnan(dbs{1}.(series{i})(cbands_rng(1)-1))
        shaded_plot(dcbands.(series{i})(cbands_rng), cbands_rng, opt.confidence_levels, ...
                    'interp',false);
      else
        shaded_plot(dcbands.(series{i})(cbands_rng), cbands_rng, opt.confidence_levels, ...
                    'interp',false,'first_value',dbs{1}.(series{i})(cbands_rng(1)-1));
      end
      hold on;
    end
    for j = 1:length(dbs)
      jj = rem(j-1,2)+1;
      plot(opt.report_plot_range, dbs{j}.(series{i}),...
           'LineWidth',st.line.lineWidth{jj},...
           'LineStyle',st.line.lineStyle{jj},...
           'Color',st.line.color{jj}); hold on;
    end
    grid;
    axis tight;
    box on;
    title(desc.(series{i}));
  end
  g = grfun.bottomlegend(labs, 'location','SouthOutside','orientation', ...
                         'horizontal');
  p = get(g, 'position');
  set(g, 'position',[p(1),p(2)+0.03,p(3),p(4)]);
  x.userfigure(tit, fh);
  
%--  x.figure(tit, 'subplot', [nx, ny], 'style', st, ...
%--           'legend',false,'range',opt.report_plot_range,...
%--           'visible',false);
%--  for i = 1:length(series)
%--    zline = abs(ss.(series{i})) < 1e-5;
%--    x.graph(desc.(series{i}),'zeroline',zline, 'postprocess',...
%--            ['x=grfun.bottomlegend(' mss ');p=get(x,''position'');set(x,''position'',[p(1),p(2)+0.04,p(3),p(4)]);']);
%--    x.highlight('', opt.report_plot_range(1):opt.forecast_range(1)-1);
%--    for j = 1:length(dbs)
%--      x.series(labs{j}, dbs{j}.(series{i}));
%--    end
%--  end
end


% create a shaded plot
%
% author: Ondra Kamenik
%
% y      variable timeseries including the bands. It must have an even
%        number of columns, the middle one is the median. The rows
%        correspond to periods.
% rng    the range of the plot plot and data y. Length of the range must
%        be the same as a number of rows in y
% clev   confidence levels for example [0.90 0.70 0.50 0.30 0.1].
%        in this case y must have 11 timeseries
% interp if true, interpolation is done, otherwise one color areas are
%        plotted, default is true 
% white  shade used for zero density (max white), defaults to 0.8
% black  shade used for density=1 (max black), defaults to 0.2
%
% the function returns the axes handle

function ah = shaded_plot(y, rng, clev, varargin)

  % get options
  optnames = {varargin{1:2:end}};
  optvals = {varargin{2:2:end}};
  opt = cell2struct(optvals, optnames, 2);
  if ~isfield(opt, 'black')
    opt.black = 0.2;
  end
  if ~isfield(opt, 'white')
    opt.white = 0.8;
  end
  
  % if first val, augment y and rng
  if isfield(opt, 'first_value')
    y = [repmat(opt.first_value,1,size(y,2)); y];
    rng = rng(1)-1 : rng(end);
  end
  
  colormap(repmat([opt.white:-(opt.white-opt.black)/100:opt.black]',1,3));
  
  if any(any((isnan(y))))
    error('Time series data include NaN');
  end
  if size(y,1) ~= length(rng)
    error('Incompatible size of data and range');
  end
  if size(y,2) ~= length(clev)*2+1
    error('Incompatible number of confidence levels wrt the data');
  end
  % dat2grid is not supported in the newer IRIS, use dat2dec with 'centre'
  % option to be compatible with default of the plot function
  x = dat2dec(rng,'centre');
  %x = dat2grid(rng);%use dat2grid because dat2dec converts 2009Q3 to 2009.5, 2009Q4 to 2009.75 and so on, which is not compatible with plot(tseries)
  xx = [x';x(end:-1:1)']; 
  ddens = [(1-clev) 1 (1-clev(end:-1:1))];
  np = length(x);
  
  for i=1:size(y,2)-1
    tmp = y(:,i+1);% we need to do this because y can be timeseries
    yy = [y(:,i); tmp(end:-1:1)];
    if isfield(opt, 'interp') && ~ opt.interp
      ii = i;
      if ii > length(clev)
        ii = ii + 1;
      end
      patch(xx, yy, ceil(ddens(ii)*100), 'LineStyle','none');
    else
      patch(xx, yy, ceil(ddens(i)*100)*ones(2*np,1), 'LineStyle','none');
    end
  end
  hold on;
  
  ah = gca;
end
