% Read the model, solve it, and create model documentation
%
%

function readmodel(self)
  
  % get round options
  opt = self.get_round_options();
  
  % create  structure with model switches, linear flags,
  % and model names
  models = struct([]);
  %- model for history or the default model for all
  if ~isstruct(opt.model_linear)
    models(1).switches = opt.model_switches;
    models(1).linear = opt.model_linear;
    models(1).name = '';
    models(1).fname = '';
    models(1).options = {};
    if isfield(opt, 'model_options')
      models(1).options = opt.model_options;
    end
  else
    flds = {'history', opt.round_scenarios{:}};
    names = {'', opt.round_scenarios{:}};
    for i = 1:length(flds)
      models(i).switches = opt.model_switches.(flds{i});
      models(i).linear = opt.model_linear.(flds{i});
      models(i).name = names{i};
      models(i).fname = ['_' names{i}];
      models(i).options = {};
      if isfield(opt, 'model_options')
        models(i).options = opt.model_options.(flds{i});
      end
    end
    models(1).fname = '';
  end
  
  % get all *.model files in the directory
  modeldir = [self.dir filesep 'model'];
  d = dir(modeldir);
  f = {d.name};
  modfiles = {};
  for i = 1:length(f)
    if strcmp(f{i}(max(1,end-5):end), '.model');
      modfiles{end+1} = [modeldir filesep f{i}];
      disp(sprintf('%-6s %s', ['read:'], modfiles{end}));
    end
  end
  
  % go through all the models, parse, solve and save them
  for i=1:length(models)
    %- calculate parameters
    f = self.fetch_file_name('run', 'set_param');
    [ptmp, n, e] = fileparts(f);
    ptmp = pathincr(ptmp);
    clear(n);
    p = eval([n '(models(i).switches);']);
    %- parse and create model file
    m = model([modeldir filesep self.prefix 'model.model'],...
              'linear', models(i).linear, ...
              'assign', models(i).switches,...
              models(i).options{:});
    m = assign(m, p);
  
    %- solve
    if models(i).linear
      m = solve(m);
      m = sstate(m);
    else
      if opt.use_sirius
        tmp = sirmexmod('iris_model',m);
        [lev, tnd, ~] = tmp.sdm_handle.get_values();
        ss = dbbatch(lev, '$0', 'lev.$0 + tnd.$0*sqrt(-1)');
        m = assign(m, ss);
        m = solve(m);
      else 
        m = sstate(m, 'growth', true, 'blocks', true);
        m = solve(m);
      end
    end

    %- report
    x = report.new(['Report on the model structure: ' models(i).name]);
    %-- model files
    for j = 1:length(modfiles)
      x.modelfile(['Model file ' num2str(j) ' for round ', self.name],...
                  modfiles{j}, m);
    end
    x.pagebreak();
    %-- steady state table
    endovars = get(m,'xlist');
    com = get(m, 'comments');
    islog = get(m, 'log');
    ad = cell(length(endovars), 5);
    for j = 1:length(endovars)
      ad{j,1} = endovars{j};
      ad{j,2} = com.(endovars{j});
      if islog.(endovars{j}), ad{j,3} = 'true'; else ad{j,3} = 'false'; end
      ad{j,4} = real(m.(endovars{j}));
      ad{j,5} = imag(m.(endovars{j}));
    end
    x.array('Steady state levels and trends', ad,...
            'heading',{'Variable','Description','Log?','Level','Trend'},...
            'long',true);
    %-- publish
    x.publish(self.fetch_file_name('write','model_report',models(i).fname),...
              'display',false);

    %- save model
    save(self.fetch_file_name('write','model_mat',models(i).fname),'m');
    
    %- write model xml for gui
    % first, get frequency of model
    if ~isfield(opt,'history_range')
        error('Please specify history_range in round_options before continuing.');
    else
        [~,~,frq] = dat2ypf(opt.history_range(end));
    end
    write_xml(gimi(m),self.fetch_file_name('write','model_xml',models(i).fname),...
        'freq',frq);
  end
end
