% Special ad-hoc report for Doug Laxton displaying:
% 1. historical loss function
% 2. contributions to the loss function
%
% options:
%    pie       YoY inflation in the loss function
%    pietar    YoY inflation target in the loss function

function special_report_lossfunction(self, varargin)

  % options
  opthis = struct(varargin{:});
    
  % set up a few things
  bsmodel = 'optbaseline';

  % get round options
  opt = self.get_round_options();
  hrng = opt.history_range(1):opt.forecast_range(1)-1;

  % get the model and historical database
  a = load(self.fetch_file_name('read','model_mat',['_' bsmodel]));
  dhist = dbload(self.fetch_file_name('read', 'scenario_history_mean', bsmodel));
  
  % calculate the loss and contributions
  apiedev = a.m.w_D4L_CPIXFE_DEV*(dhist.(opthis.pie)-dhist.(opthis.pietar))^2;
  agap = a.m.w_L_GDP_GAP*dhist.L_GDP_GAP^2;
  adrs = a.m.w_RS*(dhist.RS-dhist.RS{-1})^2;
  aloss = apiedev+agap+adrs;
  
  % variable descriptions
  desc = get(a.m, 'comments');
  
  % report
  x = report.new(['Special report on historical ex-post loss function ' ...
                  self.name]);
  % graph of the loss function and components
  x.figure('Ex-post loss funtion and its components',...
           'subplot', [2, 2],'legend',false,...
           'range',hrng,'zeroline',true,...
           'visible',false);
  x.graph('Actual period contribution to the Loss Funtion (index)');
  x.series('',aloss);
  x.graph(['Deviation of ' desc.(opthis.pie)]);
  x.series('',dhist.(opthis.pie)-dhist.(opthis.pietar));
  x.graph(desc.L_GDP_GAP);
  x.series('',dhist.L_GDP_GAP);
  x.graph(['Change of ' desc.RS]);
  x.series('', dhist.RS-dhist.RS{-1});
  
  % decomposition of the loss function
  fh = figure('visible','off');
  bar(resize([apiedev, agap, adrs], hrng),'stack');hold on;
  plot(hrng, aloss, 'LineWidth', 2);
  legend('Inflation dev','Output gap','Interest rate change',...
         'orientation','horizontal','location', ...
         'southoutside');
  x.userfigure('Decomposition of the ex-post loss function',...
               fh);

  % publish
  x.publish(self.fetch_file_name('write','special_report_lossfunction'),...
            'display',false);
end
