function write_xml(self, outfile, varargin)

  % process opt
  opt = struct(varargin{:});
  %- model name
  modname = self.model_name();
  if isfield(opt, 'modelname'), modname = opt.modelname; end
  %- symboldesc
  symboldesc = struct();
  if isfield(opt, 'symboldesc'), symboldesc = opt.symboldesc; end
  %- autoexogenise  
  autoexogenise = self.model_autoexogenise();
  if isfield(opt, 'autoexogenise')    
    flds = fieldnames(opt.autoexogenise);
    for i = 1:length(flds)
      autoexogenise.(flds{i}) = opt.autoexogenise.(flds{i});
    end
  end
  
  % open file and start writing
  fid = fopen(outfile, 'w');
  if isfield(opt,'freq')
      switch opt.freq
          case 1
              freqstr = 'y';
          case 4
              freqstr = 'q';              
          case 12
              freqstr = 'm';
      end
    fprintf(fid, '<model freq="%s">\n',freqstr);
  else
    fprintf(fid, '<model>\n');  
  end
  fprintf(fid, '<modelname>%s</modelname>\n', modname);
  fprintf(fid, '<modeldesc />\n');
  fprintf(fid, '<symbols>\n');
  % symbols
  [endo, exo, params] = self.model_symbols();
  %- split label into label/unit
  for i = 1:length(endo)
      tmp_label = endo(i).label;
      [tmp_label,tmp_unit] = strtok(tmp_label,'(');
      endo(i).label = tmp_label(1:end-1); % do not include whitespace at end
      endo(i).unit = regexprep(tmp_unit,{'(',')'},'');
  end
  for i = 1:length(exo)
      tmp_label = exo(i).label;
      [tmp_label,tmp_unit] = strtok(tmp_label,'(');
      exo(i).label = tmp_label(1:end-1); % do not include whitespace at end
      exo(i).unit = regexprep(tmp_unit,{'(',')'},'');
  end
  %- endos
  for i = 1:length(endo)
    if endo(i).is_additive, flag='true'; else flag='false'; end;
    write_symbol(fid, endo(i), symboldesc, 'endo', flag);
  end
  %- exos
  for i = 1:length(exo)
    write_symbol(fid, exo(i), symboldesc, 'shock');
  end
  %- parameters
  for i = 1:length(params)
    write_symbol(fid, params(i), symboldesc, 'parameter');
  end
  fprintf(fid, '</symbols>\n');
  % equations
  eqs = self.model_equations();
  fprintf(fid, '<equations>\n');
  for i = 1:length(eqs)
    fprintf(fid, '  <equation>\n');
    fprintf(fid, '    <label>Equation %04d</label>\n', i);
    expr = eqs{i}.rhs;
    if ~strcmp(eqs{i}.lhs,'0')
      expr = [expr ' - (' eqs{i}.lhs ')'];
    end
    fprintf(fid, '    <expression>%s</expression>\n', expr);
    fprintf(fid, '  </equation>\n');
  end
  fprintf(fid, '</equations>\n');
  % values
  [endoval, trendval, paramval] = self.model_values();
  fprintf(fid, '<values>\n');
  write_struct(fid, '  ', 'levels', endoval);
  write_struct(fid, '  ', 'trends', trendval);
  write_struct(fid, '  ', 'parameters', paramval);
  fprintf(fid, '</values>\n');
  
  % autoexogenise
  fprintf(fid, '<autoexogenise>\n');
  flds = fieldnames(autoexogenise);
  for i = 1:length(flds)
      fprintf(fid, ' <pair endo="%s" shock="%s" />\n', flds{i}, autoexogenise.(flds{i}));
  end
  fprintf(fid, '</autoexogenise>\n');
  
  fprintf(fid, '</model>\n');
  fclose(fid);  
  
end

function write_symbol(fid, sym, desc, type, varargin)

  % merge info from desc
  if isfield(desc, sym.name)
    if isfield(desc.(sym.name), 'label')
      sym.label = desc.(sym.name).label;
    end
    if isfield(desc.(sym.name), 'unit')
      sym.unit = desc.(sym.name).unit;
    end
    if isfield(desc.(sym.name), 'latexAlias')
      sym.latexAlias = desc.(sym.name).latexAlias;
    end
  end
  
  % printout the info
  if nargin == 5
    flag = varargin{1};
    fprintf(fid, '  <symbol name="%s" type="%s" isAdditive="%s">\n',...
            sym.name, type, flag);
  else
    fprintf(fid, '  <symbol name="%s" type="%s">\n',...
            sym.name, type);
  end
  if isfield(sym,'label')
    fprintf(fid, '    <label>%s</label>\n', sym.label);
  else
    fprintf(fid, '    <label />\n');
  end
  if isfield(sym,'unit')
    fprintf(fid, '    <unit>%s</unit>\n', sym.unit);
  else
    fprintf(fid, '    <unit />\n');
  end
  if isfield(sym,'latexAlias')
    fprintf(fid, '    <latexAlias>%s</latexAlias>\n', sym.latexAlias);
  else
    fprintf(fid, '    <latexAlias />\n');
  end
  
  fprintf(fid, '  </symbol>\n');
end

function write_struct(fid, indent, name, s)
  
  fprintf(fid, '%s<%s>\n', indent, name);
  flds = fieldnames(s);
  for i = 1:length(flds)
    fprintf(fid, '%s  <value name="%s">%.13g</value>\n', indent, ...
            flds{i}, s.(flds{i}));
  end  
  fprintf(fid, '%s</%s>\n', indent, name);
end