% abstract class for dynamic model information

classdef gmi

  methods (Abstract)
    
    % get model name
    % 
    % r = self.model_name();
    % 
    %  self   object of this gmi class
    %  r      returned string
    r = model_name(self)
    
    % return model symbols, labels, units. etc
    %
    % [endo, exo, params] = self.model_symbols();
    %
    %  self   object of this gmi class
    %  endo   struct array with fields:
    %            name*        string of the variable name
    %            is_additive* true/false
    %            label        string describing the variable
    %            unit         string describing the unit
    %            latexAlias   string for LaTeX rendering of the variable
    %                         name
    %  exo    struct array with fields:
    %            name*        string of the variable name
    %            label        string describing the variable
    %            unit         string describing the unit
    %            latexAlias   string for LaTeX rendering of the variable
    %                         name
    %  params struct array with fields:
    %            name*        string of the variable name
    %            label        string describing the variable
    %            unit         string describing the unit
    %            latexAlias   string for LaTeX rendering of the variable
    %                         name
    %
    % field* means that the field is mandatory
    [endo, exo, params] = model_symbols(self)

    % return model equations
    %
    %   self   object of this gmi class
    %   r      returned cell array of structs with fields
    %             lhs   string fro the left hand side
    %             rhs   string fro the right hand side
    r = model_equations(self)
    
    % return values of the variables and parameters
    %
    %   self   object of this gmi class
    %   levels struct with fields endo variables and values steady state
    %          values
    %   trends struct with fields endo variables and values the trends
    %   params struct with fields parameters and values the param values
    [levels, params, trends] = model_values(self)
    
  end
  
  methods
    
    % write XML file with the model information
    %
    % write_xml(self, ...)
    % 
    % self            object of this gmi class
    % modelname       string for the model name
    % symboldesc      struct of structs, for selected symbols (field)
    %                 there can be label, unit, latexAlias 
    % autoexogenise   struct to add to autoexogenise info from the model
    write_xml(self, outfile, varargin)
    
  end
end