function dbs = cz_historical_databases(c, rng)

% check that rng is monthly
% if datfreq(rng(1)) ~= 12
% 	error('Function can provide historical databases for monthly frequencies only');
% end

% now we emulate that we return historical databases, in fact we just
% chop the data and for not complete quarters, we make biased averages
% to emulate some nowcasting with errors

% load the monthly and quartely databases
dm		= dbload(c.fetch_file_name('read','data_monthly_csv'));
dq		= dbload(c.fetch_file_name('read','data_quarterly_csv'));
dy		= dbload(c.fetch_file_name('read','data_yearly_csv'));
dtunes	= dbload(c.fetch_file_name('read','scenario_tunes','baseline'));
opts	= c.get_round_options();

if datfreq(rng(1)) == 12
	% now we emulate that we return historical databases, in fact we just
	% chop the data and for not complete quarters, we make biased averages
	% to emulate some nowcasting with errors
	dbs = {};
	for dt = rng
		dbs{end+1} = emulate_monthly_data(dm, dq, dt, forfix);
	end
elseif datfreq(rng(1)) == 4
	dbs = {};
	for dt = rng
		dbs{end+1} = emulate_quarterly_data(dm, dq, dy, dtunes, dt, opts);
		disp(rng(end)-dt)
	end
else
	error('Function can provide historical databases for monthly and quarterly frequencies only');
end

function d = emulate_quarterly_data(dm, dq, dy, dtunes, dt, opts)

d = cz_observed_data(opts.model_switches, dm, dq, dy);
d = dboverlay(d,dtunes);

vars = fieldnames(d);
for i = 1:length(vars)
	d.(vars{i}) = resize(d.(vars{i}), get(d.(vars{i}),'start'):dt);
end
d.OBS_L_GDP = resize(d.OBS_L_GDP, get(d.OBS_L_GDP,'start'):dt-1);

function d = emulate_monthly_data(dm, dq, dt)

dtq = convert(dt, 4);
pos = dt-convert(dtq,12)+1;

% quarterly GDP and components
gdplag = -2;
if pos > 1
	gdplag = -1;
end

tmp = convert(x12(100*log(dm.cpi_imf_10)),'q');
d.OBS_L_CPI = resize(dq.CPI, get(dq.CPI,'start'):dtq-1);
d.OBS_L_CPI(dtq) = pos/3*dq.L_CPI(dtq)+(3-pos)/3*(dq.L_CPI(dtq-1)+0.5);

d.OBS_L_CPIXFE = resize(dq.L_CPIXFE, get(dq.L_CPIXFE,'start'):dtq-1);
d.OBS_L_CPIXFE(dtq) = pos/3*dq.L_CPIXFE(dtq)+(3-pos)/3*(dq.L_CPIXFE(dtq-1)+0.5);

d.OBS_L_GDP = resize(dq.GDP, get(dq.GDP,'start'):dtq+gdplag);

d.OBS_RS = resize(dq.RS, get(dq.RS,'start'):dtq-1);
d.OBS_RS(dtq) = pos/3*dq.RS(dtq) + (3-pos)/3*dq.RS(dtq-1);

d.OBS_L_S = resize(exp(dq.L_S/100), get(exp(dq.L_S/100),'start'):dtq-1);
d.OBS_L_S(dtq) = pos/3*exp(dq.L_S(dtq)/100) + (3-pos)/3*exp(dq.L_S(dtq-1)/100);

% external sector
d.OBS_NC_RS_RW = resize(dq.RS_RW, get(dq.RS_RW,'start'):dtq-1);
d.OBS_NC_RS_RW(dtq) = pos/3*dq.RS_RW(dtq) + (3-pos)/3*dq.RS_RW(dtq-1);
d.OBS_DOTA_CPI_RW = resize(dq.DOTA_CPI_RW, get(dq.DOTA_CPI_RW,'start'):dtq-1);
d.OBS_DOTA_CPI_RW(dtq) = pos/3*dq.DOTA_CPI_RW(dtq)+(3-pos)/3*(2);
d.OBS_DOTA_CPIE_RW = resize(dq.DOTA_CPIE_RW, get(dq.DOTA_CPIE_RW,'start'):dtq-1);
d.OBS_DOTA_CPIE_RW(dtq) = pos/3*dq.DOTA_CPIE_RW(dtq)+(3-pos)/3*(2);
d.OBS_L_GDP_GAP_RW = resize(dq.L_GDP_GAP_RW, get(dq.L_GDP_GAP_RW,'start'):dtq-1);
d.OBS_L_GDP_GAP_RW(dtq) = pos/3*dq.L_GDP_GAP_RW(dtq-1)+(3-pos)/3*dq.L_GDP_GAP_RW(dtq);
