% report history and add graphs and tables to the report object
%
%

function x = az_report_history(x, m, s, rng, dobs, dtunes, d)

% merge parameters of the models into d
p = get(m, 'parameters');
d = dboverlay(d, p);

% styles
st1 = struct();
st1.line.lineWidth = {1.5,1.5};
st1.line.lineStyle = {'-','--'};
st1.line.color = {[0,0,0.9],[0,0,0]};
st1.line.marker = {'none','none'};

st2 = st1;
st2.line.lineWidth = {1,1};

st3 = st1;
st3.line.lineStyle = {'-','--','-.'};
st3.line.Color = {[0,0,0.9],[0,0,0],[0.9,0,0]};
st3.line.Marker = {'none','none','none'};

sto =struct();
sto.line.linewidth = {2};
sto.line.color = {[0,0.9,0],[0,0,0.9],[0.9,0,0],[0.4,0.4,0]};

%%%%%%%%%
%% observed data
% x.figure('Observed data','subplot',[2,2],'range',rng,...
%     'style',st2);
% x.graph('Real GDP (100*log)');
% x.series('',d.obs_l_y);
% x.graph('Headline CPI (100*log)');
% x.series('',d.obs_l_cpi);
% x.graph('Policy Rate (%, p.a.)');
% x.series('',d.obs_ip);
% x.graph('AZN per USD');
% x.series('',exp(d.obs_l_azn_usd/100));
% 
% %%%%%%%%%
% % inflation
% x.figure('Inflation',...
%     'style', st2, 'legend',true, 'range', rng);
% x.graph('Headline Inflation');
% x.series('QoQ @ar', d.dl_cpi);
% x.series('YoY'    , d.d4l_cpi);

%%%%%%%%%
% trends
x.figure('Levels and Trends (dashed)', 'subplot',[2,2],...
    'style', st2, 'legend', false, 'range', rng);
x.graph('GDP (100*log)');
x.series('', d.l_y);
x.series('', d.l_y_tnd);
x.graph('Real Exchange Rate (100*log)');
x.series('', d.l_z);
x.series('', d.l_z_tnd);
x.graph('Real Interest Rate (%, p.a.)');
x.series('', d.r);
x.series('', d.r_tnd);
x.graph('Headline Inflation and Target (%, YoY)');
x.series('', d.d4l_cpi);
x.series('', d.dl_cpi_tar);

% gaps
x.figure('Gaps', 'subplot',[2,2],...
    'style', st2, 'legend', false, 'range', rng,'zeroline',true);
x.graph('GDP Gap (percent)');
x.series('', d.l_y_gap);
x.graph('Real Exchange Rate Gap (percent)');
x.series('',d.l_z_gap);
x.graph('Real Interest Rate Gap (pp)');
x.series('', d.r_gap);
x.graph('Headline Inflation Deviation from Target (pp)');
x.series('', d.d4l_cpi - d.dl_cpi_tar);

% % growth
% x.figure('Growth Rates with Trend Growth (dashed)', 'subplot',[2,2],...
%     'style', st2, 'legend', false, 'range', rng);
% x.graph('GDP (%, YoY)');
% x.series('', d.d4l_y);
% x.series('', d.dl_y_tnd);
% x.graph('Real Exchange Rate (%, YoY)');
% x.series('', diff(d.l_z,-4));
% x.series('', d.dl_z_tnd);
% 
% % interest rate
% x.figure('Interest Rate', 'subplot',[2,2],...
%     'style', st2, 'legend', false, 'range', rng);
% x.graph('Nominal Interest Rate (percent p.a.)','legend',true);
% x.series('',d.i,'legend',NaN);
% x.series('neutral',d.i_tnd);
% x.graph('Real Interest Rate (percent p.a.)','legend',true);
% x.series('',d.r,'legend',NaN);
% x.series('trend',d.r_tnd);
% x.graph('Real Interest Rate Gap (pp)','zeroline',true);
% x.series('', d.r_gap);
% x.graph('Country risk premium (percent p.a.)','zeroline',true);
% x.series('', d.prem);


% RER
x.figure('Exchange Rate', 'subplot',[2,2],...
    'style', st2, 'legend', false, 'range', rng);
x.graph('Nominal Exchange Rate vs. USD (100*log)');
x.series('', d.l_azn_usd);
x.graph('Real Exchange Rate (100*log)');
x.series('', d.l_z);
x.series('', d.l_z_tnd);
x.graph('Real Exchange Rate Gap (percent)');
x.series('',d.l_z_gap);
x.graph('REER vs Real Oil Price','legend',true,'legendLocation','ne');
x.series('',[d.l_z-d.l_z(qq(2014,1))... 
             -(d.l_qoil-d.l_qoil(qq(2014,1)))],...
    'legend',{'REER','(-1)*Oil'});

%% Decompositions
% IS curve
fh = figure('visible','off');
u = [...
	+ d.c1_l_y_gap*d.e_l_y_gap, ...
  + d.c2_l_y_gap*d.l_y_gap{-1}, ...
  - d.c3_l_y_gap*d.c1_rmci*d.r_gap, ...
  + d.c3_l_y_gap*(1-d.c1_rmci)*d.l_z_gap, ...
  + d.c4_l_y_gap*d.l_y_foreign_gap, ...
  + d.c5_l_y_gap*d.l_qoil_gap, ...
  + d.shock_l_y_gap];
barcon(rng, u); hold on;
p = plot(rng, [d.l_y_gap d.l_y_gap],'LineWidth',1.5); grid on;
set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
legend('Backward','Forward','RIR gap', 'REER gap',...
	'Foreign Demand','Oil Price','Shock',...
	'location','Southoutside','orientation','horizontal');
xlabel(' ')
x.userfigure('Real Output Gap (%)', fh);

% phillips curve
fh = figure('visible','off');

u = [...
	+ d.c1_dl_cpi*d.e_dl_cpi,... 
  + (1-d.c1_dl_cpi-d.c2_dl_cpi-d.c3_dl_cpi)*d.dl_cpi{-1},... 
  + d.c2_dl_cpi*d.dl_cpi_imp,...
  + d.c3_dl_cpi*d.dl_cpi_food_imp,...
  + d.c4_dl_cpi*(1-d.c1_rmc-d.c2_rmc)*d.l_y_gap,...
  + d.c4_dl_cpi*(d.c1_rmc+d.c2_rmc)*d.l_z_gap,...
  + d.c4_dl_cpi*d.c2_rmc*d.l_qfood_gap,...  
	+ d.shock_dl_cpi];
barcon(rng, u); hold on;
p = plot(rng, [d.dl_cpi d.dl_cpi]); grid on;
set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
legend('Expectations','Lag','Imported','Imported Food',...
	'Output gap','REER Gap', 'Food Gap','Shock',...
	'location','Southoutside','orientation','horizontal');
xlabel(' ')
x.userfigure('Phillips Curve (% QoQ @ar)', fh);

% MP rule
fh = figure('visible','off');
u = [d.c1_i*d.i{-1},...
	+ (1-d.c1_i)*d.i_tnd, ...
	+ (1-d.c1_i)*d.c2_i*d.d4l_cpi_dev, ...
  + (1-d.c1_i)*d.c3_i*d.l_y_gap, ...
	+ d.shock_i];
barcon(rng, u); hold on;
p = plot(rng, [d.i d.i],'LineWidth',1.5); grid on;
set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
legend('Lag','Neutral','Inflation Deviation','Output Gap','Shock',...
	'location','Southoutside','orientation','horizontal');
xlabel(' ')
x.userfigure('Policy Rate (%, p.a.)', fh);

% % FX Rate rule
% fh = figure('visible','off');
% subplot(311);
%   u = [d.dl_azn_usd_tar,...
%        (-4)*d.c3_l_azn_usd*d.l_z_gap{-1}];
%   barcon(rng, u); hold on;
%   % p = plot(rng, [d.l_azn_usd-d.l_azn_usd{-1} d.l_azn_usd-d.l_azn_usd{-1}],'LineWidth',1.5); grid on;
%   set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
%   legend('FX target depr.','RER gap',...
%     'location','Southoutside','orientation','horizontal');
%   title('Policy FX Rule Depreciation');  
% subplot(312);
%   u2 = [4*(d.e_l_azn_usd-d.l_azn_usd{-1}),...
%       -d.i,...
%        d.i_us,...
%        d.prem,...
%       -4*d.c4_l_azn_usd*d.l_qoil_gap];
%   bar_plot(u2, rng); hold on;
%   % p = plot(rng, [d.l_azn_usd-d.l_azn_usd{-1} d.l_azn_usd-d.l_azn_usd{-1}],'LineWidth',1.5); grid on;
%   set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
%   legend('Expected depr.','Domestic rate','US rate','Prem','Oil Gap',...
%     'location','Southoutside','orientation','horizontal');
%   title('UIP Implied Depreciation, pct QoQ');
% subplot(313);
%   u = [d.c1_l_azn_usd*(d.dl_azn_usd_tar-4*d.c3_l_azn_usd*d.l_z_gap{-1}),...
%        (1-d.c1_l_azn_usd)*sum(u2,2),...
%         4*d.shock_l_azn_usd];
%   barcon(rng, u); hold on;
%   p = plot(rng, [d.dl_azn_usd d.dl_azn_usd],'LineWidth',1.5); grid on;
%   set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
%   legend('FX Rule','UIP','Shock',...
%     'location','Southoutside','orientation','horizontal');
%   title('Actual depreciation, pct QoQ');
%   % xlabel(' ')
%   x.userfigure('FX Rule Decomposition', fh);
  


% UIP TO BE DONE
% fh = figure('visible','off');
% u = [d.c1_e_l_azn_usd*(d.l_azn_usd-d.l_azn_usd),...
% 	(1-d.c1_e_l_azn_usd)*(2*d.dl_azn_usd_tar - d.dl_azn_usd),...
% 	4*d.l_azn_usd];
% barcon(rng, u); hold on;
% p = plot(rng, [d.ip-d.i_us d.ip-d.i_us],'LineWidth',1.5); grid on;
% set(p,{'linewidth'},{2.5;1.5},{'color'},{'w';'k'});
% legend('Country Premium','Exp. Deprec','Trend Deprec','UIP Shock',...
% 	'location','Southoutside','orientation','horizontal');
% xlabel(' ')
% x.userfigure('Interest Rate Differential (p.p., p.a.)', fh);

% FROM MODEL
% % l_azn_usd =  c1_l_azn_usd*(
% %                             l_azn_usd{-1} 
% %                           + dl_azn_usd_tar/4 
% %                           - c3_l_azn_usd*(l_z_gap{-1})) + ...
% %             (1-c1_l_azn_usd)*(
% %                             e_l_azn_usd + (-ip + i_us + prem)/4  
% %                             - c4_l_azn_usd*l_qoil_gap) 
% %           + shock_l_azn_usd;
% 
% e_l_azn_usd = c1_e_l_azn_usd     * l_azn_usd{+1} 
%             + (1-c1_e_l_azn_usd) * (l_azn_usd{-1} + 2/4*(dl_azn_usd_tar) );
% 
            

%% foreign relative prices
% x.figure('Real Price to respective CPI: Actual and Trend',...
% 	'subplot', [2,1], 'style', st3, 'legend',false, 'range', rng);
% %- real world energy price
% x.graph('Real Oil Price World (100*log)');
% x.series('', d.l_qoil);
% x.series('', d.l_qoil_tnd);
% %- real world food price
% x.graph('Real Food Price World (100*log)');
% x.series('', d.l_qfood);
% x.series('', d.l_qfood_tnd);


%% shocks
% elist_d = {'SHK_L_GDP_GAP',...
% 	'SHK_DLA_CPIXFE',...
% 	'SHK_L_RP_GAP',...
% 	'SHK_RS',...
% 	'SHK_L_S',...
% 	'SHK_UNR_GAP',...
% 	'SHK_NAIRU',...
% 	'SHK_D_NAIRU',...
% 	'SHK_DLA_RP_BAR',...
% 	'SHK_CTYPREM',...
% 	'SHK_DLA_GDP_BAR',...
% 	'SHK_D4L_CPI_TAR'};
% elist_f1 = {'SHK_DLA_QOIL_BAR',...
% 	'SHK_L_QOIL_GAP',...
% 	'SHK_DLA_QFOOD_BAR',...
% 	'SHK_L_QFOOD_GAP',...
% 	'SHK_DLA_GDP_BAR_US',...
% 	'SHK_L_GDP_GAP_US',...
% 	'SHK_RS_US',...
% 	'SHK_RR_BAR_US',...
% 	'SHK_DLA_CPI_US',...
% 	'SHK_DLA_CPIXFE_US'};
% elist_f2 = {'SHK_L_GDP_GAP_EU',...
% 	'SHK_L_Z_GAP_EU',...
% 	'SHK_L_GDP_GAP_JA',...
% 	'SHK_L_Z_GAP_JA',...
% 	'SHK_L_GDP_GAP_CH',...
% 	'SHK_L_Z_GAP_CH',...
% 	'SHK_L_GDP_GAP_EA7',...
% 	'SHK_L_Z_GAP_EA7',...
% 	'SHK_L_GDP_GAP_LA6',...
% 	'SHK_L_Z_GAP_LA6',...
% 	'SHK_L_GDP_GAP_RC6',...
% 	'SHK_L_Z_GAP_RC6'};
% fh = dbplot(d,elist_d,rng);
% set(fh,'visible','off');
% x.userfigure('Domestic Shocks', fh);
% fh = dbplot(d,elist_f1,rng);
% set(fh,'visible','off');
% x.userfigure('Foreign Shocks', fh);
% fh = dbplot(d,elist_f2,rng);
% set(fh,'visible','off');
% x.userfigure('Foreign Shocks (cont.)', fh);

end


function bar_plot(u, prng)

% divide to positive and negative
u = u(prng);
upos = u;
upos(find(u < 0)) = 0;
uneg = u;
uneg(find(u > 0)) = 0;
upos = tseries(prng, upos);
uneg = tseries(prng, uneg);
% plot it
bar(upos,'stack'); hold on;
bar(uneg,'stack'); hold on;
end
