% function which returns various options for the various steps

function opt = az_round_options()

opt = struct();
opt.use_sirius = false;
opt.stochastic_simulations = false;

% model parsing
opt.round_scenarios = {'baseline'};

opt.model_linear    = struct(...
  'history',      true,...
  'baseline',     true);

opt.model_switches  = struct(...
  'history',      {{}},...
  'baseline',     {{}});

opt.model_options   = struct(...
  'history',      {{}},...
  'baseline',     {{}});

% IRFs
opt.irfs_periods = 20;
opt.irfs_shocks = {'shock_l_y_gap', 'shock_dl_cpi', ...
	'shock_l_azn_usd', 'shock_i', 'shock_dl_cpi_tar','shock_dl_z_tnd','shock_l_z_tnd','shock_dl_y_tnd'};
opt.irfs_vars = {{'l_y_gap','dl_cpi','d4l_cpi','i','l_z','l_z_gap','r_gap','l_cpi','l_azn_usd'}};

% vardecomp
opt.vardecomp_periods = 20;
opt.vardecomp_contribs = 6;
opt.vardecomp_variables = {'d4l_cpi','l_azn_usd','i','l_y_gap','d4l_y'};

% hist decomp
opt.histdecomp_variables = {...
	'dl_cpi', 'l_y_gap', 'i', 'dl_azn_usd','d4l_cpi'};
opt.histdecomp_groups = {...
	{'Initial conditions'}, ...
	{'shock_dl_cpi'},...
	{'shock_l_y_gap'},...
	{'shock_l_azn_usd'},...
	{'shock_l_qoil_gap', 'shock_dl_qoil_tnd', 'shock_l_qfood_gap', 'shock_dl_qfood_tnd'},...
	{'shock_i_us', 'shock_r_us_tnd', 'shock_dl_cpi_us', 'shock_l_y_us_gap', ...
   'shock_dl_cpi_ez','shock_l_y_ez_gap', 'shock_dl_cpi_ru', 'shock_dl_cpi_ru_tar', 'shock_l_y_ru_gap', ...
   'shock_dl_usd_eur', 'shock_dl_usd_rub', 'shock_dl_z_ru_us_tnd'},...
	{'shock_i','shock_dl_cpi_tar'},...
	{'shock_dl_z_tnd','shock_dl_y_tnd','shock_prem'}...
    };
opt.histdecomp_grnames = {...
	'Initial Conditions',...
	'Supply Shocks',...
	'Demand Shocks',...
	'FX rule Shock',...
	'Commodity Price Shocks', ...
	'Foreign Shocks',...
  'Monetary policy Shocks',...
  'Trends'};
opt.histdecomp_range = qq(1995,1):qq(2015,4); % Used in shock decomposition

% round specific
opt.use_kalman = true;
opt.history_range = qq(2000,1):qq(2014,4); 
opt.forecast_range = qq(2015,1):qq(2017,4);
opt.history_report_range = qq(2000,1):qq(2014,4); % startdate = minrange

% reporting (forecast)
opt.report_table_range = qq(2013,1):opt.forecast_range(end);
opt.report_plot_range = qq(2013,1):opt.forecast_range(end);
opt.report_table_col1width = '75mm';

% decomposition
opt.decomp_contribs = 6;
opt.decomp_variables = {'dl_cpi','i','l_y_gap','d4l_y','l_azn_usd'};

% historical forecasts
opt.histfcast_periods = 12;
opt.histfcast_vars = {...
	'i', ...
	'dl_cpi', 'd4l_cpi',...
	'l_y_gap', 'dl_y', 'd4l_y', ...
	'l_azn_usd', 'dl_azn_usd', ...
	'l_z_gap'};
opt.histfcast_nahead = [0, 1, 2, 4, 8, 12];
opt.histfcast_show_variable = {'dl_cpi'};
opt.histfcast_range_left = 0;
opt.histfcast_plot_hist = 0;

% additional
opt.rs_maturities = struct('i', 1/360, 'i_us', 1/360);

end
