function p = az_set_param(s)

p = struct();

%%%%%%%%%%%%%%%%%%%%%%
% Behavioral equations
%%%%%%%%%%%%%%%%%%%%%%
% IS curve
p.c1_l_y_gap = 0.10; % expected
p.c2_l_y_gap = 0.70; % lag
p.c3_l_y_gap = 0.09; % RMCI
p.c4_l_y_gap = 0.30; % foreign demand
p.c5_l_y_gap = 0.03; % oil price gap

p.c1_rmci    = 0.30; % weight on the REER gap vs the RIR gap

% Phillips curves
p.c1_dl_cpi  = 0.55;   % expected inflation
p.c2_dl_cpi  = 0.075;  % imported inflation
p.c3_dl_cpi  = 0.25;  % imported food
p.c4_dl_cpi  = 0.15;   % real marginal costs

p.c1_rmc     = 0.15;    % weight of REER gap in RMC
p.c2_rmc     = 0.20;    % weight of food gap in RMC

% Exchange rate
p.c1_l_azn_usd   = 0.85; % the degree of FX control
p.c2_l_azn_usd   = 0.00; % dl_azn_usd{-1} 
p.c3_l_azn_usd   = 0.05; % l_z_gap{-1}
p.c1_e_l_azn_usd = 0.50; % forward-lookiness of ex. rate expectations 
p.c4_l_azn_usd   = 0.027;% effect of qoil_gap (to get appreciation in reaction to oil price shock)

% Interest rate
p.c1_i           = 0.75; % interest rate smoother
p.c2_i           = 0.90; % weight on the inflation deviation
p.c3_i           = 0.02; % weight on output gap
p.c4_i           = 0.00; % weight on FX deprec. deviation

%%%%%%%%%%%%%%%%%%%%%%
% Other parameters
%%%%%%%%%%%%%%%%%%%%%%

% Persistence
p.c1_dl_z_tnd     = 0.90;
p.c2_dl_z_tnd     = 0.03;
p.c1_r_tnd        = 0.70;
p.c1_prem         = 0.90; 
p.c1_dl_cpi_tar   = 0.9999;
p.c1_dl_y_tnd     = 0.90;
p.c2_dl_y_tnd     = 0.02;

%%%%%%%%%%%%%%%%%%%%%%
% Steady states
%%%%%%%%%%%%%%%%%%%%%%
p.ss_dl_cpi   = 0;
p.ss_dl_y_tnd = 0;
p.ss_dl_z_tnd = 0;

p.ss_prem     = 0;

%%%%%%%%%%%%%%%%%%%%%%
% Standard deviations of shocks
%%%%%%%%%%%%%%%%%%%%%%
p.std_shock_l_y_gap        = 0.70;
p.std_shock_dl_y_tnd       = 0.20;
p.std_shock_l_azn_usd      = 2.00;
p.std_shock_ip             = 1.50;
p.std_shock_prem           = 0.50;
p.std_shock_dl_z_tnd       = 1.00;
p.std_shock_dl_cpi         = 3.00;
p.std_shock_dl_cpi_tar     = 0.80;
p.std_shock_r              = 0.01;
p.std_shock_dl_azn_usd_tar = 1.00;

%%%%%%%%%%%%%%%%%%%%%%
% External sector and commodities
%%%%%%%%%%%%%%%%%%%%%%

p.ss_dl_cpi_us      = 2.3;% 2;
p.ss_dl_cpi_ez      = 1.9;
p.ss_dl_qfood_tnd   = 0;
p.ss_dl_qoil_us_tnd = 5 - p.ss_dl_cpi_us;
p.ss_dl_usd_eur     = p.ss_dl_cpi_us - p.ss_dl_cpi_ez;
p.ss_r_us_tnd       = 1;
p.ss_dl_cpi_ru      = 100*log(1+5.0/100);
% trade weight
p.c1_us             = 0.6; 
p.c1_ru             = 0.1; 
p.c1_i_us           = 0.85;
p.c1_r_us_tnd       = 0.924;
p.c1_dl_cpi_us      = 0.38;
p.c1_l_y_us_gap     = 0.86;
p.c1_l_y_ez_gap     = 0.80;
p.c1_dl_cpi_ez      = 0.70;
p.c1_dl_cpi_ru      = 0.80;
p.c1_l_y_ru_gap     = 0.80;
p.c1_dl_z_ru_us_tnd = 0.7;
p.c1_dl_cpi_tar_ru  = 0.95;
p.c_w_tot_ru        = 0.65/2;
% Food and Oil Price
p.c1_dl_qfood_tnd   = 0.90; 
p.c1_l_qfood_gap    = 0.75;
p.c1_dl_qoil_tnd    = 0.90;
p.c1_l_qoil_gap     = 0.75;
p.ss_dl_z_ru_us     = -p.c_w_tot_ru*p.ss_dl_qoil_us_tnd;
p.ss_dl_qoil_tnd    = p.ss_dl_qoil_us_tnd + p.ss_dl_cpi_us - ((1 - p.c1_ru - p.c1_us)*(p.ss_dl_cpi_ez + p.ss_dl_usd_eur) + p.c1_ru*(-p.ss_dl_z_ru_us + p.ss_dl_cpi_us) + p.c1_us*p.ss_dl_cpi_us);


% Foreign sector shocks
p.std_shock_l_qoil_gap   = 17.4;
p.std_shock_dl_qoil_tnd  = 5.7;
p.std_shock_l_qfood_gap  = 6.24;
p.std_shock_dl_qfood_tnd = 3.2;

p.std_shock_i_fed_us     = 0.95;
p.std_shock_r_fed_us_tnd = 0.45;
p.std_shock_dl_cpi_us    = 1.56;
p.std_shock_prem_3lib_us = 0.2607;
p.std_shock_prem_6lib_us = 0.2753;
p.std_shock_l_y_us_gap   = 0.5548;

p.std_shock_dl_cpi_ez    = 0.86;
p.std_shock_dl_usd_eur   = 17.5406;
p.std_shock_l_y_ez_gap   = 0.6578;

p.std_shock_dl_cpi_ru    = 3.0;
p.std_shock_dl_usd_rub   = 22;
p.std_shock_l_y_ru_gap   = 2.1392;

end
