% constructs approximated terms of goods trade for ZTN
% 
% dq: country input db
% dq_us: input db with US-model based data


function out = construct_atot(dq, dq_us, trade_cat_names, p_w, suffix)

%ztn_construct_atot(dq,dm,p_w,trade_cat_names,'dqAdd',dq_add);

  if nargin == 4
    suffix = '_sa12';
  end

  out = struct();

  % starting and ending date
  sdate = qq(1996,1);
  edate = get(dq_us.cpi_us,'last');
  %[syear,squarter,tmp_]=dat2ypf(sdate);
  %[eyear,equarter,tmp_]=dat2ypf(edate);
  
  pList=fieldnames(p_w); 
  for i=1:numel(pList)
    if isfield(dq_us,['l_' pList{i}])  
        out.p.(['l_' pList{i}]) = dq_us.(['l_' pList{i}]);
    else
        error(['Time series l_' pList{i} ' not found in the US database']);
    end   
  end
   
  ex_categ=strcat('ex_comm_',regexp(trade_cat_names.ex, '\|','split'));
  im_categ=strcat('im_comm_',regexp(trade_cat_names.im, '\|','split')); 
  ex_categ=strcat(ex_categ,suffix);
  im_categ=strcat(im_categ,suffix);
  
  ex_tot_sa12_hpf=tseries(get(dq.(['ex_comm_tot' suffix]),'range'),0);
  im_tot_sa12_hpf=tseries(get(dq.(['im_comm_tot' suffix]),'range'),0);
  
  % interpolate the trade items and put to dq its hpfiltered trend
  dq = dbbatch(dq,'$0_hpf','hpf(dq.$0,Inf,''lambda'',4000)','namelist',ex_categ+im_categ);
  
  % calculate total trades based on hpfiltered series
  for i=1:length(ex_categ)
    ex_tot_sa12_hpf= ex_tot_sa12_hpf+dq.([ex_categ{i} '_hpf']);
  end

  for i=1:length(im_categ)
    im_tot_sa12_hpf= im_tot_sa12_hpf+dq.([char(im_categ{i}) '_hpf']);
  end
  
% weights ext. trade components  
  w_x_names=strcat('w_x_',regexp(trade_cat_names.ex, '\|','split'));
  for i=1:length(ex_categ)
    out.(w_x_names{i})= dq.([ex_categ{i} '_hpf'])/ex_tot_sa12_hpf;
  end

  w_m_names=strcat('w_m_',regexp(trade_cat_names.im, '\|','split'));
  for i=1:length(im_categ)
    out.(w_m_names{i})= dq.([char(im_categ{i}) '_hpf'])/im_tot_sa12_hpf;
  end

  %%% end of Tunisia-specific code %%%  
  % weights of prices for all categories
  list_ = w_x_names+w_m_names;

  for i = 1:length(list_)
    rng_ = get(out.(list_{i}),'first');
    if rng_ > sdate-1
     out.(list_{i})(sdate-1:rng_) =  out.(list_{i})(rng_);
    end
    rng_ = get(out.(list_{i}),'last');
    if rng_ < edate
     out.(list_{i})(rng_+1:edate) =  out.(list_{i})(rng_);
    end
  end

  
  l_atot0=0;
  l_atot1=0;
  % calculate weights for prices and atot
  for i=1:length(pList)
    out.(['pw_x_' pList{i}])=0; 
    out.(['pw_m_' pList{i}])=0; 
    tmp.(['pw_' pList{i} '1'])=0;
    if isfield(p_w.(pList{i}),'ex')
      fd_ex=fieldnames(p_w.(pList{i}).ex);
      for j=1:length(fd_ex)
        out.(['pw_x_' pList{i}])=out.(['pw_x_' pList{i}])+    out.(['w_x_' fd_ex{j}])*p_w.(pList{i}).ex.(fd_ex{j});
        tmp.(['pw_' pList{i} '1'])=tmp.(['pw_' pList{i} '1'])+out.(['w_x_' fd_ex{j}]){1}*p_w.(pList{i}).ex.(fd_ex{j});
      end
    end
    if isfield(p_w.(pList{i}),'im')
      fd_im=fieldnames(p_w.(pList{i}).im);
      for j=1:length(fd_im)
        out.(['pw_m_' pList{i}])=out.(['pw_m_' pList{i}])+out.(['w_m_' fd_im{j}])*p_w.(pList{i}).im.(fd_im{j});
        tmp.(['pw_' pList{i} '1'])=tmp.(['pw_' pList{i} '1'])-out.(['w_m_' fd_im{j}]){1}*p_w.(pList{i}).im.(fd_im{j});
      end
    end
    out.(['tot0_' pList{i}])=(out.(['pw_x_' pList{i}])-out.(['pw_m_' pList{i}]))*out.p.(['l_' pList{i}]);
    out.(['tot1_' pList{i}])=tmp.(['pw_' pList{i} '1'])*out.p.(['l_' pList{i}]);
    
    l_atot0=l_atot0+out.(['tot0_' pList{i}]);
    l_atot1=l_atot1+out.(['tot1_' pList{i}]);
  end
  
  sdate=get(l_atot0,'first');
  out.l_atot = tseries(sdate,100*log(100));
  
  for i = sdate+1:edate
      out.l_atot(i) = out.l_atot(i-1)-l_atot1(i-1) + l_atot0(i);
  end
end