function  fceval(of)
close all
clc

o      = ogiroot();

% set the list of countries you want to evaluate the forecast for
cclist = {'am','by','cd','ge','gt','ht','ke','kg','kz',...
         'md','mm','mn', 'mz', 'ng', 'ruf', 'tj', 'za', 'zm' };

% cclist = {'eg'};
     
%set the variables and their descriptions
vars    = {'d4l_cpi','d4l_y','d4l_fx', 'i'};
vardesc = {'CPI inflation (YoY)', 'Real GDP growth (YoY)', 'Exchange rate growth (LCY/USD, YoY)', 'Nominal interest rate (pp)'};

% set country specific names for policy rate: needed if observed policy rate is not called "i" or "ip" 
voc_ip.mn = 'i_wa';  
voc_ip.ng = 'i_tb';

% set country specific names for exchange rate: needed if a country has
% more LCY/USD exchange rate
voc_fx.by = 'byr_usd';


%delete unwanted timeseries from graphs
C    = cell(numel(cclist),1);
C(:) = {ones(1, 1)};
rm   = cell2struct(C, cclist', 1);
for j = 1: numel(vars)
    dlt.(vars{j}) = rm;
end

dlt.i.mm = NaN;
dlt.d4l_y.kg = NaN;
dlt.d4l_y.mz = NaN;

%set the periods (in quarters) you want to evaluate the forecast for 
fah     = [1 2 4 8];% 12];

%get the list of published fcasts
uopt    = o.xopt.opt;
[svnStat, pubList] = system([uopt.svnbinary, ' list ', ...
    uopt.publishsvnurl]);
if svnStat
    error('SVN repository is unavailable, cannot get up-to-date published_results!');
end

%set empty matrices for storing the results
FE_M    = nan(numel(vars),numel(cclist),numel(fah),30);
NFE_M   = nan(numel(vars),numel(cclist),numel(fah),30);

%get and create output dir
p       = mfilename('fullpath');
idx     = regexp(p, '\ogi\');
outdir  = [p(1:idx-1) '\wogi\fceval\results\'];  
mkdir(outdir);
%% generate frontpage

out  = '\textbf{General Information} \\[5mm]';
out  = [out, 'This report measures the quality of OGResearch forecasts across time and countries. The results are aggregated and reported using simple charts and tables.\\[5mm]'];
  
tmp1 = ['\textbf{There are ' num2str(numel(cclist)) ' countries included in this report:} \\'];
for i    = 1:numel(cclist);
    tmp1 = [tmp1, upper(cclist{i}) ', '];
end
  
out  = [out, tmp1, '\\[5mm]'];
out  = [out,'\textbf{Description of the criteria used}:  \\ [5mm]'];

if strcmpi(of,'rmse');
    out  = [out,'The relative RMSE is calculated by diving the average forecast error by the average error of a naive prediction (variable stays at the level equal to average of the last two years). Lower values indicate better forecast performance and values below one indicate that the macro forecast outperforms the naive prediction.  \\ [5mm]'];
elseif  strcmpi(of,'hr');
    out  = [out,'A hit is defined as a forecast that correctly predicted an increase or a decline in the forecasted variable. The hit ratio expresses shares of hits to the overall number of forecasts. The closer is the result to one, the more precise forecasts are. \\ [5mm]'];
end

tmp1 = '\textbf{Variables included:} \\[5mm] We evaluate our forecasts for the following variables: \\';
  for i    = 1:numel(vardesc);
      tmp1 = [tmp1, vardesc{i} ', '];
  end
out  = [out, tmp1, '\\[5mm]'];


tmp2 = 'Latest available forecast for a given country: \\';
tmp3 = 'Available forecasts for a given country (dates and total number):\\';

%% Get the historical fcasts for each country in the list

for cidx = 1:numel(cclist)
    
%find latest published (full) fcast for the country  
cc         = cclist{cidx};    
pubList_cc = regexp(pubList,['\<',cc,'\d{6}\>'],'match')';
idr        = pubList_cc{end};

%check out historical fcasts and hard data (use 'fc' action for this)
[str, ff, lf] = run_action(o, idr, 'fc');
eval(str);   

tmp_list = fieldnames(usereses);
db_list  = tmp_list(2:end-1);  

%create strings that contains all necessary information about historical
%fcastd
tmp2 = [tmp2, upper(cc) ':~', dat2char(lf, 'dateformat','YYYYFP') ',  ' ];
tmp3 = [tmp3, [upper(cc) ':~', ...
    dat2char(ff, 'dateformat','YYYYFP') '--' dat2char(lf, 'dateformat','YYYYFP')  '~(' num2str(numel(db_list)) ')'   ',  '] ];   


%load observed data and make necessary transforamtions
sm_dir  = get_results_directory(o, idr, 'sm');
tmp     = load([sm_dir '\results\' cc '_algo_attributes.mat']);
aa      = tmp.action_attribs;
tmp     = load([sm_dir '\results\' cc '_smoothed_db.mat']);
dbobs   = tmp.hist.mean;
dbobs   = dbclip(dbobs,aa.firstHist:aa.firstFore-1);

%find the name of fx rate 
fx = regexp(fieldnames(dbobs),([ cc(1:2) '\w_usd']),'match');
FX = fx(~cellfun('isempty',fx));
FX = FX{1}{1};

if isfield(voc_fx, cc);
    FX = voc_fx.(cc);
end

dlt.(['d4l_' FX]).(cc) = dlt.d4l_fx.(cc);



%find policy rate
if isfield(dbobs, 'ip');
    dbobs.i = dbobs.ip;
end

if isfield(voc_ip, cc);
    dbobs.i = dbobs.(voc_ip.(cc));
end


%make sure that dataset contains YoY growth rates
dvars   = {'cpi','y',FX};
d4vars  = cell(1,numel(dvars));  
for dv = 1:numel(dvars)
    d4vars{dv} = ['d4l_' dvars{dv}];
    if isfield(dbobs, [dvars{dv}])
    dbobs.(d4vars{dv}) = diff(log(dbobs.([dvars{dv}]))*100,-4)*dlt.(d4vars{dv}).(cc);
    elseif isfield(dbobs, ['l_' dvars{dv}])
    dbobs.(d4vars{dv}) = diff(dbobs.(['l_' dvars{dv}]),-4)*dlt.(d4vars{dv}).(cc);
    end
end 


dbobs.d4l_fx = dbobs.(['d4l_' FX]);
dbobs.i = dbobs.i*dlt.i.(cc);
%% load fcasts and calculate hits/fcast errors for each round

%load smoother results
for i  = 1:numel(db_list) 
    
pref = [usereses.(char(db_list{i})).path filesep 'results' filesep cc '_'];
if exist([pref 'smoothed_db.mat'],'file')
   tmp = load([pref 'smoothed_db.mat']);
   if isfield(tmp,'fore')
      dbs = tmp.fore.mean;
   else
      dbs = tmp.meanFore;
   end
end  

fd     = regexp(usereses.(char(db_list{i})).idr, '\d{6}','match');
fcdate = qq(str2double(fd{1,1}(1:4)),floor((str2double(fd{1,1}(5:6))-1)/3));  


%make sure to have YoY growth rates in the fcast database  
for dv = 1:numel(dvars)
    if isfield(dbs, [dvars{dv}])
        dbs.(d4vars{dv}) = diff(log(dbs.([dvars{dv}]))*100,-4);
    elseif isfield(dbs, ['l_' dvars{dv}])
        dbs.(d4vars{dv}) = diff(dbs.(['l_' dvars{dv}]),-4);
    end
end    
dbs.d4l_fx = dbs.(['d4l_' FX]);

%find policy rate
if isfield(dbs, 'ip');
    dbs.i = dbs.ip;
end

if isfield(voc_ip, cc);
    dbs.i = dbs.(voc_ip.(cc));
end
 
%calculate hits/fcast error (we will use the matrix
%only, but the struct is a very useful way to store the hits/errors)
for v = 1:length(vars);
    for tind = 1:numel(fah)
        fa = fah(tind);
        if strcmpi(of,'rmse');
           [NFE.(vars{v}).(cc).(['Q' num2str(fa)])(i), FE.(vars{v}).(cc).(['Q' num2str(fa)])(i)] = ferror(dbs,dbobs,fa,vars{v}, fcdate );
           NFE_M(v,cidx, tind,i) = NFE.(vars{v}).(cc).(['Q' num2str(fa)])(i);
        elseif strcmpi(of,'hr');
           FE.(vars{v}).(cc).(['Q' num2str(fa)])(i) = numhit(dbs,dbobs,fa,vars{v}, fcdate );
           NFE = [];
        end
        FE_M(v,cidx, tind,i) = FE.(vars{v}).(cc).(['Q' num2str(fa)])(i);        
    end    
end
    
    
end    

end

out  = [out, tmp2, '\\[5mm]', tmp3, '\\[5mm]'];
save([ outdir 'FE_' lower(of)  '.mat'],'FE','FE_M','NFE','NFE_M','cclist', 'fah', 'out', 'of', 'vardesc', 'tmp3');

%% Creating the report

%set options to true if you want to show distribution by country (cc), by periods
%ahead (ff)
plotcc = true;
plotff = true;

if strcmpi(of,'rmse');
    x = report('heading', 'Relative RMSE distributions for different forecast horizonts', 'color', true);
elseif strcmpi(of,'hr');
    x = report('heading', 'Hit ratio distributions for different forecast horizonts', 'color', true);
end

x = begintext(x);
x = tex(x,out);
x = endtext(x);


HR = zeros(size(FE_M,1),size(FE_M,2),size(FE_M,3));
O  = zeros(size(FE_M,1),size(FE_M,2),size(FE_M,3));

for j = 1: size(FE_M,1)  

%calculating average hits/mean square errors
for k = 1:size(FE_M,2);
    HR(j,k,:) = nanmean_(reshape(FE_M(j,k,:,:),size(FE_M,3),[]),2);
    O(j,k,:)  = nanmean_(reshape(NFE_M(j,k,:,:),size(FE_M,3),[]),2);
end

%calculate relative RMSEs/collect hitrates  
if strcmpi(of,'rmse');    
    dbf     = reshape(sqrt(HR(j,:,:)./O(j,:,:)), size(HR,2),size(HR,3));
elseif strcmpi(of,'hr');
    dbf     = reshape(HR(j,:,:), size(HR,2),size(HR,3));
end

cc2p    = cclist(~isnan(max(dbf,[],2)));
dbf     = dbf(~isnan(max(dbf,[],2)),:);
%% Plot distribution by horizont

neworderf = cell(numel(fah),2);
for fa = 1:numel(fah)
    if fah(fa)<4
    neworderf(fa,1:2) = {[num2str(fah(fa)) 'Q' ] fa};
    else
    neworderf(fa,1:2) = {[num2str(fah(fa)/4) 'Y' ] fa};
    end
end


if plotff
x = newpage(x);
x = opengraph(x);

%create boxplots
if numel(cclist) == 1
    dbf(2,:) = NaN;
end
[T] = bplot(dbf,'linewidth',2);

%set axis' properties
xlabel('Forecast horizon','FontSize',14);


if strcmpi(of,'rmse');
    set(gca,'Xtick',1:numel(fah),'XtickL',neworderf(:,1),'Fontsize',14, 'linewidth',2);
    ylabel('RMSE relative to naive forecast (2-year average)','FontSize',14);
    legend(T,'location','NW');
    x = tag(x, [vardesc{j}] , 'fontsize', 'Large', 'visible', false,'footnote','Values below one indicate that the macro forecast outperforms the naive prediction'); 
elseif strcmpi(of,'hr');
    set(gca,'Xtick',1:numel(fah),'XtickL',neworderf(:,1),'Fontsize',14,'ylim',[0-1e-2 1+1e-2], 'linewidth',2);
    ylabel('Hit ratio','FontSize',14);
    legend(T,'location','SW');
    x = tag(x, [vardesc{j}] , 'fontsize', 'Large', 'visible', false, 'footnote','The closer to one, the better our forecast');
end
    
 
x = getgraph(x);
end


%% Plot distribution by countries

neworderc = cell(numel(cc2p),2);
for fa = 1:numel(cc2p)
    neworderc(fa,1:2) = {upper(cc2p{fa}) fa};
end

if plotcc  
x   = newpage(x);
x   = opengraph(x);

%create boxplots
if numel(cclist) == 1
    dbf = dbf(1,:);
end

[T] = bplot(dbf','linewidth',2);

%set axis' properties
xlabel('Countries','FontSize',14);

if strcmpi(of,'rmse');
    set(gca,'Xtick',1:numel(cc2p),'XtickL',neworderc(:,1),'Fontsize',13,'linewidth',2);
    ylabel('RMSE relative to naive forecast (2-year average)','FontSize',14);
    legend(T,'location','NW');
    x = tag(x, [vardesc{j}] , 'fontsize', 'Large', 'visible', false,'footnote','Values below one indicate that the macro forecast outperforms the naive prediction');
elseif strcmpi(of,'hr');
    set(gca,'Xtick',1:numel(cclist),'XtickL',neworderc(:,1),'Fontsize',13,'ylim',[0-1e-2 1+1e-2],'linewidth',2);
    ylabel('Hit ratio','FontSize',14);
    legend(T,'location','SW');
    x = tag(x, [vardesc{j}] , 'fontsize', 'Large', 'visible', false,'footnote','The closer to one, the better our forecast');
end
    
  
x = getgraph(x);
end


%% Create table with RMSEs/hit rates
x = beginmatrix(x);
x = data(x, dbf','fontsize',5);
x = colnames(x,neworderc(:,1),'fontsize',5);
x = rownames(x,neworderf(:,1),'fontsize',5);

if strcmpi(of,'rmse');
    x = tag(x, ['Relative RMSE for '  vardesc{j}],'fontsize',10,'footnote', tmp3);
elseif strcmpi(of,'hr');
    x = tag(x, ['Hit ratios for '  vardesc{j}],'fontsize',10,'footnote', tmp3);
end

x = endmatrix(x);

end

outname = [outdir 'fc_eval_' lower(of) '.pdf'];
compile(x, outname, 'deletetex', 0);
end


%% Additional functions used

%using fc acction to get historical fcasts
function[strfunc,ff, lf]=run_action(o, idr, ida)

action = get_action(o, idr, ida);  
% set usereses
str5 = 'usereses = struct();' ;
if isfield(action,'usereses')
  xusereses = action.usereses;
  fcdl = cell(1,xmldom.num_children(xusereses));
  for i = 1:xmldom.num_children(xusereses)
    if any(strfind(xusereses(i).idr,'$last$')) % construct round id automatically by checking publised_results repo
       mc = xusereses(i).idr; 
       mc = mc(1:strfind(xusereses(i).idr,'$last$')-1);
       xusereses(i).idr = get_last_published_round(o, mc, idr);
    end
    if isfield(o.rounds, xusereses(i).idr)
       a  = get_round(o, xusereses(i).idr);
       mc = a.modelcode;
    else
       mc = round.modelcode;
    end
    if strcmp(idr, xusereses(i).idr) 
        p  = get_action_directory(o, xusereses(i).idr, xusereses(i).ida);
    else
        p  = get_results_directory(o, xusereses(i).idr, xusereses(i).ida);
    end
    
    str5 = [str5 'usereses.' xusereses(i).name ...
              ' = struct(''idr'',''' xusereses(i).idr ''','...
              '''ida'',''' xusereses(i).ida ''',' ...
              '''path'',''' p ''',' ...
              '''modelcode'',''' mc ''');' ];
    fcdl{i} = xusereses(i).idr; 
  end
  
  ff = mm(str2double(fcdl{2}(numel(mc)+1:numel(mc)+4)),str2double(fcdl{2}(numel(mc)+5:numel(mc)+6)));
  lf = mm(str2double(fcdl{end}(numel(mc)+1:numel(mc)+4)),str2double(fcdl{end}(numel(mc)+5:numel(mc)+6)));
  
end
  
strfunc = str5;
end


%function used for calculating fcast errors
function [nfe, fe] = ferror(fcdb,odb,fa, var,fcd)

try  
    fe    = (fcdb.(var)(fcd+fa) - odb.(var)(fcd+fa)).^2;      
    nfe   = (odb.(var)(fcd+fa)  - mean(odb.(var)(fcd-7:fcd))).^2;  
catch
    fe    = NaN;
    nfe   = NaN;
end
end

%function used for calculating hits
function a = numhit(fcdb,odb,fa, var,fcd)

if isnan(odb.(var)(fcd+fa))|| isnan (fcdb.(var)(fcd+fa))
    a = NaN;
elseif  sign(fcdb.(var)(fcd+fa)-fcdb.(var)(fcd)) == sign(odb.(var)(fcd+fa)-odb.(var)(fcd));
    a = 1; 
else 
    a = 0;
end
end