function out = norm_cdf_skew(x,mu,sigma,gamma)
% Function to calculate CDF of two-piece normal distribution.
% Input can be a vector.
% 
% Inputs:   
%            x .......... quantile (scalar)
%           mu .......... mean of the distribution
%        sigma .......... standard error of the distribution
%        gamma .......... skeweness parameter from (-1;1); in practice,
%                         keep it in (-0.97,0.97), becase higher skeweness
%                         produces very nigh tail numbers that are hard to
%                         operate with numerically

if sigma < 0
  error('Standard error must be non-negative!')
end
if (gamma > 1) || (gamma < -1)
  error('Skeweness parameter must be between -1 and 1!')
end


% call auxiliary funtion for each point of x
out = zeros(size(x));
for iter = 1:length(x)
  out(iter) = norm_cdf_skew_aux(x(iter),mu,sigma,gamma);
end


end

