function out = norm_cdf_skew_aux(x,mu,sigma,gamma)
% Function to calculate CDF of two-piece normal distribution.
% Input can only be a single point, not a vector, because we numerically
% integrate.
% 
% Inputs:   
%            x .......... quantile (scalar)
%           mu .......... mean of the distribution
%        sigma .......... standard error of the distribution
%        gamma .......... skeweness parameter from (-1;1); in practice,
%                         keep it in (-0.97,0.97), becase higher skeweness
%                         produces very nigh tail numbers that are hard to
%                         operate with numerically

  npoints = 10e4; % number of points over which to integrate

  % set lower bound for integration interval
  bound = -15*sigma;
  
  while bound >= x % if requested quantil is below the lower bound, shift the bound closer to -Inf
    bound = bound*2;
  end
  
  width = (x - bound)/(npoints-1); % width of intervals in the grid
  if abs(bound) < 10e4
    while width > 1
        npoints = npoints*2;
        width = (x - bound)/(npoints-1);
    end
  end

  aux = linspace(bound,x,npoints); % grid over which we'll integrate
  
  if width <= 0
    error('Width of interval for integration is not positive, it is %2.5f. Sthing is wrong.',width)
  end
  
  aux2 = norm_pdf_skew(aux,mu,sigma,gamma); % get the probabilities 
  
  % two-piece normal has troubles for x == mu: approximate
  index = find(aux == mu);
  aux2(index) = norm_pdf_skew(mu+0.5*width,mu,sigma,gamma); %#ok<FNDSB>
  
  % the integral is sum of the function values times the width of the grid
  % interval
  
  out = sum(aux2)*width;
end

