function cf = norm_confbands_skew(clev, myStd, gamma)
%function calculates confidence intervals specified in clev for normal
%distributions with zero mean and std. deviations stored in vector
%forecastStd.
%
%Inputs: clev ....................... confidence levels for example 
%                                     [0.90 0.70 0.50 0.30 0.1]. In this 
%                                     case cf will have 11 timeseries for
%                                     each field (variable) in forecastStd
%        myStd       ................ vector of std. deviations of normal
%                                     distributions
%        gamma       ................ vector of skeweness parameters for
%                                     two-piece normal distribution (the same lenght as myStd)


  nStd  = length(myStd);
  nClev = length(clev);

  % create the confbands
  cf = zeros(nStd, 2*nClev+1);

  % iterate over std's and fill in the quantiles
  for iter = 1:length(myStd)
    if myStd(iter) == 0
      cf(iter,:) = 0;
    else
      tmp                       = norm_inv_skew((1-clev)/2  , 0, myStd(iter), gamma(iter));
      cf(iter, 1:nClev)         = tmp;
      cf(iter, end-nClev+1:end) = fliplr(norm_inv_skew(1-(1-clev)/2  , 0, myStd(iter), gamma(iter)));
    end
  end
  
  
end