function out = norm_inv_skew(x,mu,sigma,gamma)
% Numerically approcimating inverse of the two-piece normal CDF by method of
% halving intervals. The input can be a vector.
%
% Inputs: 
%            x .......... density
%           mu .......... mean of the distribution
%        sigma .......... standard error of the distribution
%        gamma .......... skeweness parameter from (-1;1); in practice,
%                         keep it in (-0.97,0.97), becase higher skeweness
%                         produces very nigh tail numbers that are hard to
%                         operate with numerically
%
% Outputs: 
%          out .......... quantile of two-piece normal distribution that
%                         corresponds to density x

if sigma < 0
  error('Standard error must be non-negative!')
end
if (gamma > 1) || (gamma < -1)
  error('Skeweness parameter must be between -1 and 1!')
end

out = zeros(size(x));

% use auxiliary function for each point
for iter = 1:length(x)
  out(iter) = norm_inv_skew_aux2(x(iter),mu,sigma,gamma);
end


end

