function out = norm_inv_skew_aux2(x,mu,sigma,gamma)
% Numerically approximating inverse of the two-piece normal CDF by method of
% halving intervals.
%
% Inputs: 
%            x .......... density
%           mu .......... mean of the distribution
%        sigma .......... standard error of the distribution
%        gamma .......... skeweness parameter from (-1;1); in practice,
%                         keep it in (-0.97,0.97), becase higher skeweness
%                         produces very nigh tail numbers that are hard to
%                         operate with numerically

if abs(sigma) > 10e12
    tol = 0.01;
else
    tol        = 1e-10;0.0000001;   % desired tolerance
end
upperBound = 1e4*sigma;   % upper bound for initial search
lowerBound = -1e4*sigma;  % lower bound for initial search
difference = Inf;         % initial difference between desired and obtained density


iter = 0; % count number if iterations, too many mean sthing is wrong
yy   = 0;

while abs(difference) > tol

  if iter > 1e4
    if upperBound == lowerBound % the bounds are not wide enough => widen them
      if yy < x % density is lower then required  => we have to go right (to obtain higher density)
        upperBound = upperBound*10;
      else      % density is hogher then required => we have to go left (to obtain lower density)
        lowerBound = lowerBound*10;
      end
      iter = iter-10; 
      continue
    else % we have a different problem, keyboard and investigate
      fprintf('Algorithm stuck after %4.1f iterations, keyboarded intentionally.\n',iter)
      fprintf('x = % 2.5f\n',x)
      fprintf('mu = % 2.5f\n',mu)
      fprintf('sigma = % 2.5f\n',sigma)
      fprintf('gamma = % 2.5f\n',gamma)
      
      keyboard      
    end
  end
  
  y  = (upperBound+lowerBound)/2; % set y to the middle of interval
  
  yy = norm_cdf_skew_aux(y,mu,sigma,gamma); % evaluate density at y

  if yy > x % density higher than we require => decrease the upper bound
    upperBound = y;
  else      % density lower than we require  => increase the lower bound
    lowerBound = y;
  end
  
  
  
  
  difference = yy-x; % calculate if we're close enough
  iter = iter+1;
  
  % for debug
  % [lowerBound upperBound];

end
  
out = y;

end

