function out = norm_pdf_skew(x,mu,sigma,gamma)
% Probability distribution function of two-piece normal distribution.
% 
% Inputs:   
%            x .......... quantile 
%           mu .......... mean of the distribution
%        sigma .......... standard error of the distribution
%        gamma .......... skeweness parameter from (-1;1); in practice,
%                         keep it in (-0.97,0.97), becase higher skeweness
%                         produces very nigh tail numbers that are hard to
%                         operate with numerically

E = -0.5/sigma^2 * ((x - mu).^2 + gamma*((x-mu)./(abs(x-mu)).*(x-mu).^2));
S = exp(E) ./ (sqrt(2*pi) .* sigma);

A = 2 / ( (1/sqrt(1-gamma)) + (1/sqrt(1+gamma)));

out = A*S;
end

